## Title: gnipactor.py
## Author: Scott O. Nelson (SirGnip)
## License: LGPL (c) 2007 Scott O. Nelson
## Description: a collection of general purpose actors

'''
Module with some simple actor-like objects.  Useful for debugging or simple graphic
work.  Currently, most of these actor's are simple geometric shapes with lifetimes.
'''
import gniputil
import gnipmath

class LifetimeActor(gniputil.Actor):
    def __init__(self, lifetime):
        self.lifetimeRemaining = lifetime
    def loop(self):
        self.lifetimeRemaining -= 1
    def canreap(self):
        return(self.lifetimeRemaining <= 0.0)
    def reap(self):
        pass
    
##class Dot(cLifetimeActor):
##    def __init__(self, position, color, lifetime):
##        cLifetimeActor.__init__(self, lifetime)
##        self._position = position
##        self._color = color
##
##    def paint(self, screen):
##        pygame.display.get_surface().set_at(self._position.AsIntTuple(), self._color)
##

class Circle(LifetimeActor):
    def __init__(self, position, radius, color, width, lifetime):
        LifetimeActor.__init__(self, lifetime)
        self.position = position
        self.radius = radius
        self.color = color
        self.width = width

    def paint(self, screen):
        screen.circle(self.color, self.position, self.radius, self.width)


class Circle2(gniputil.Actor):
    '''
    radiusRange - the start/end radii of the circle as it animates thru over its lifetime
        (ex: (0, 5) or (25, 19))
    colorRange - the start/end colors the circle will animate thru over its lifetime
        (ex:  ((255, 0, 0), (0, 0, 0)     # red fades to black
              ((0, 0, 255), (255, 0, 0))  # blue to red
    '''
    
    def __init__(self, position, radiusRange, colorRange, width, lifetime):
        self.position = position
        self.radiusRange = radiusRange
        self.colorRange = colorRange
        self.width = width
        self.lifetimeLimit = lifetime
        self.lifetimeElapsed = 0

    def paint(self, screen):
        u = float(self.lifetimeElapsed) / self.lifetimeLimit
        radius = gnipmath.Lerp(self.radiusRange[0], self.radiusRange[1], u)
        color = gniputil.ColorLerp(self.colorRange[0], self.colorRange[1], u)
        screen.circle(color, self.position, radius, self.width)

    def loop(self):
        self.lifetimeElapsed += 1
        
    def canreap(self):
        return(self.lifetimeElapsed >= self.lifetimeLimit)

    
##class Line(cLifetimeActor):
##    def __init__(self, pos1, pos2, color, lifetime):
##        cLifetimeActor.__init__(self, lifetime)
##        self._pos1 = pos1
##        self._pos2 = pos2
##        self._color = color
##
##    def paint(self, screen):
##        pygame.draw.line(pygame.display.get_surface(), self._color, self._pos1.AsIntTuple(), self._pos1.AsIntTuple())
##    
##class Rect(cLifetimeActor):
##    def __init__(self, rect, color, lifetime):
##        cLifetimeActor.__init__(self, lifetime)
##        self._rect = rect
##        self._color = color
##
##    def paint(self, screen):
##        pygame.draw.rect(pygame.display.get_surface(), self._color, self._rect)
##
##
##


# NOTE: This requires the value generators from gnipmath        
##class PulseCircle:
##    def __init__(self, pos, radiusAnimCurve, color, width, lifetime = 0):
##        self.looping = bool(lifetime == 0)
##        self.position = pos
##        self.radiusAnimCurve = radiusAnimCurve
##        self.color = color
##        self.width = width
##        self.lifetimeRemaining = lifetime
##        self.lifetimeElapsed = 0
##        
##    def loop(self):
##        self._lifetimeRemaining -= 1
##        self._lifetimeElapsed += 1
##        
##    def paint(self, screen):
##        radius = int(self.radiusAnimCurve.Get(self._lifetimeElapsed))
##        #pygame.draw.circle(pygame.display.get_surface(), self._color, self._position.AsIntTuple(), radius)
##        screen.circle(self.color, self.position, radius, width)
##        
##    def canreap(self):
##        if self._looping:
##            return False
##        else:
##            return self.lifetimeRemaining <= 0
##    
##    def reap(self):
##        self.lifetimeRemaining = 0
##        
##               
##
