## Title: gniplevel.py
## Author: Scott O. Nelson (SirGnip)
## License: LGPL (c) 2007 Scott O. Nelson
## Description: The main "Gnip_Elite" mod implementation

import galcon
import classic
import level
import gniputil

# previously mylevel
class EliteLevelMod(level.Level_):
    '''This class was created to be able to override all the stuff I need while
    touching as little of the base mod code (level.py, main.py, etc.) as possible'''

    def copy(self):
        '''override base class version of this function'''
        return Level(self.game,self.opts,self.parent,self.data)
        
    def init(self):
        level.Level_.init(self)

        self.img = galcon.Image('eliteattack.png')
        
        # put Elite's on plaeyrs/bots home planets
        for user in self.users.values():
            if user.n == 0: # skip neutral
                user.EliteHome = None
            else:                
                # HACK! This is assuming that the planets are always in the order of the players
                user.EliteHome = self.planets.values()[user.n - 1]
            user.EliteFleet = None
            
    def loop(self):
        returnVal = classic.Level.loop(self)

        # determine if fleet carrying Elite has died (a failed attack)
        # (all other edges are handled elsewhere)
        # TODO: This logic can be moved when/if the galcon API exposes when fleets are destroyed
        for user in self.users.values():
            if user.n == 0: continue  # skip Neutral
            if user.EliteFleet and user.EliteFleet not in self.fleets.values():
                user.EliteHome = None
                user.EliteFleet = None
        return(returnVal)
        
    def paint(self, screen):
        returnVal = level.Level_.paint(self, screen)
        # draw Elite aura around Elite planets
        for user in self.users.values():
            if user.n == 0: continue
            if user.EliteHome:
                self.DrawEliteAura(screen, user.color, user.EliteHome.pos, user.EliteHome.r)
            if user.EliteFleet:
                fleetRect = user.EliteFleet.get_rect()
                fleetCenter = (fleetRect.centerx, fleetRect.centery)
                self.DrawEliteAura(screen, gniputil.ColorLighten(user.color, 200), fleetCenter, 10)
        return(returnVal)

    def hit(self,fleet,ships,p):
        '''overriding Level.hit from classic.py.  This is an exact duplicate of
        its functionality except for the Elite additions.'''
        s = p.ships
        if p.user.team == fleet.user.team:
            p.ships += ships
            # if Elite on this fleet, move to friendly planet
            # BUG: May be some issues with team games here.  ex: What happens
            # if your Elite fleet lands on a teammate's planet.  Does it die?
            # Does teammate get two elites? What happens if a my Elite land's
            # on a teammate's planet that already has an Elite?  Merge into
            # one Elite?  More than one Elite per planet?  It gets complex...
            if fleet == fleet.user.EliteFleet:
                fleet.user.EliteHome = p
                fleet.user.EliteFleet = None
        else:
            p.ships -= ships
            # If Elite fleet attacking, do extra damage and play extra particles
            if fleet == fleet.user.EliteFleet:
                p.ships -= ships  # doubles the damage
                galcon.parts_add(self.img, p.pos, 75, 5) # image, pos, radius, particles
                
            if p.ships < 0:
                # planet is conquered
                # test to see if there was an Elite that got destroyed along with the planet
                if p == p.user.EliteHome:
                    p.user.EliteHome = None
                    p.user.EliteFleet = None
                p.ships = -p.ships
                p.user = fleet.user
                # if this was an Elite fleet, move victor's Elite onto the planet
                if fleet == fleet.user.EliteFleet:
                    fleet.user.EliteHome = p
                    fleet.user.EliteFleet = None
        return p.ships-s

    
    def fleet(self,user,from_,to,ships):
        '''overriding Level.fleet from classic.py.  This is an exact duplicate of
        its functionality except for the Elite additions.'''
        if from_.user != user: return
        f = galcon.Fleet()
        f.user = user
        f.from_ = from_
        f.to = to
        f.ships = ships
        from_.ships -= ships
        self.fleets.add(f)

        # If fleet sent from planet with Elite on it, remove Elite from the
        # planet and put on the fleet
        if user.EliteHome == from_:
            user.EliteHome = None
            user.EliteFleet = f


    def DrawEliteAura(self, screen, color, pos, radius):
        colorDelta = 55
        color = gniputil.ColorDarken(color, colorDelta)
        screen.circle(color, pos, radius + 7, 3)
        color = gniputil.ColorDarken(color, colorDelta)
        screen.circle(color, pos, radius + 11, 2)
        color = gniputil.ColorDarken(color, colorDelta)
        screen.circle(color, pos, radius + 15, 2)
                


# patterned after function of same name in level.py
def Level(game,parent,opts,data=None):
    l_ = EliteLevelMod(game,parent,opts,data)
    l = galcon.Level(l_)
    l_.state = l
    return l
