## Title: bot.py
## Author: Phil Hassey
## License: Public Domain
## Description: Just a simple bot.  Feel free to do whatever with it.
##
## Description: Edits by Scott Nelson (aka SirGnip) for the Gnip_Nuke mod

import galcon
import math
import random
import gniputil
import gnipoptions


def GetPlanetsOwnedBy(level, user):
    results = []
    for p in level.planets.values():
        if p.user == user:
            results.append(p)
    return results



class Bot(galcon.Bot):
    def __init__(self,level):
        self.level = level
        
    def init(self):
        self.frame = random.randrange(0,FPS)
        self.nukeTimer = 0
        self.nukePlanet = None
        
    def loop(self):
        self.frame += 1
        if self.nukePlanet:
            self.nukeTimer += 1
            if self.nukeTimer > galcon.FPS*2.0:
                self.FireNuke()
                
        if (self.frame%galcon.FPS) != 0: return
        
        from_ = None
        for p in self.level.planets.values():
            if p.user != self: continue
            if from_ and p.ships < from_.ships: continue
            from_ = p
        if from_ == None: return
        
        self.NukeQuery()
        
        to = None
        value = -1000
        for p in self.level.planets.values():
            if p.user.team == self.team: continue
            dist = math.hypot(p.pos[0]-from_.pos[0],p.pos[1]-from_.pos[1])
            v = -p.ships + p.production - (dist / 5)
            if to and v < value: continue
            to,value = p,v
        if to == None: return
        
        self.level.fleet(self,from_,to,from_.ships*65/100)

    def NukeQuery(self):
        bestRatio = 1.0
        for nukePotential in GetPlanetsOwnedBy(self.level, self):
            myShips = 0
            enemyShips = 0
            for p in gniputil.GetPlanetsInRadius(self.level.planets, nukePotential.pos, gnipoptions.Nuke.BlastRadius):
                if p.user == self:
                    myShips += p.ships
                else:
                    if p.user.n != 0:
                        enemyShips += p.ships
            # fire nuke if there is a 3:1 ratio or better of enemy ships to bot's ships
            if enemyShips == 0:
            	continue # avoid div by 0
            ratio = float(myShips) / enemyShips

            if ratio < 1.0/3.0 and ratio < bestRatio:
                self.nukePlanet = nukePotential
                bestRatio = ratio


    def FireNuke(self):
        # fire nuke only if bot still owns the planet
        if self.nukePlanet.user == self:
            self.level.FireNuke(self.nukePlanet.pos, self)
        self.nukePlanet = None
        self.nukeTimer = 0
