## Title: gniplevel.py
## Author: Scott O. Nelson (SirGnip)
## License: LGPL (c) 2007 Scott Nelson
## Description: The main "Gnip_Nuke" mod implementation

import galcon
import classic
import level
import gniputil
import gnipactor

class NukeMod(level.Level_):
    '''This class was created to be able to override all the stuff I need while
    touching as little of the base mod code (level.py, main.py, etc.) as possible'''

    def copy(self):
        '''override base class version of this function'''
        return Level(self.game,self.opts,self.parent,self.data)
        
    def init(self):
        level.Level_.init(self)
        for user in self.users.values():
            if user.n != 0:  # Neutral player gets no nukes
                user.nukesRemaining = self.opts['numnukes']
    
        self.actorList = gniputil.ActorList()
        self.img = galcon.Image('nukeparticle.png')
        self.imgsmall = galcon.Image('nukeparticlesmall.png')
        self.nukeVFXTimer = 0
        self.nukeVFXPos = None
        self.nukeRadius = self.opts['nukeradius']
        self.damagePercent = float(self.opts['nukedamage']) / 100
        
    def loop(self):
        if self.nukeVFXPos:
            self.nukeVFXTimer += 1
        if self.nukeVFXTimer > 10:
            galcon.parts_add(self.img, self.nukeVFXPos, self.nukeRadius, 100) # large "puff" particles
            galcon.parts_add(self.imgsmall, self.nukeVFXPos, self.nukeRadius, 35) # small bits
            self.nukeVFXPos = None
            self.nukeVFXTimer = 0
            
        returnVal = classic.Level.loop(self)
        self.actorList.loop()
        return(returnVal)

    def event(self,e):
        if e.type is KEYDOWN and e.key == K_n:
            # check list of currently selected items (planets and fleets)
            if len(self.player.from_) == 1:
                selected = self.player.from_[0]
                # make sure selected item is a planet that is owned by the player
                if isinstance(selected, galcon.Planet) and selected.user == self.player:
                    self.FireNuke(selected.pos, selected.user)
        return(classic.Level.event(self, e))

    def FireNuke(self, pos, user):
        if user.nukesRemaining <= 0:
            return
        else:
            user.nukesRemaining -= 1
            
        # In order to ensure the nuke particles dont get reaped by galcon (for performance
        # reasons?), fire them after a slight delay.  When the nuke goes off, a lot of planets will
        # probably be playing their "destroyed" particle effects which can cause the nuke
        # particle effects to not fire or get cut short.
        self.nukeVFXPos = pos
        self.nukeVFXTimer = 0

        # draw effects
        radius = self.nukeRadius
        self.actorList.append(gnipactor.Circle2(pos, (0, radius+100), (user.color, (0, 0, 0)), 10, galcon.FPS*0.5))
        self.actorList.append(gnipactor.Circle2(pos, (0, radius), ((255, 255, 255), (0, 0, 0)), 10, galcon.FPS*0.5))
        self.actorList.append(gnipactor.Circle2(pos, (0, radius+50), (user.color, (0, 0, 0)), 10, galcon.FPS*2.0))
        self.actorList.append(gnipactor.Circle2(pos, (radius, 0), ((0, 0, 0), (255, 255, 255)), 10, galcon.FPS*0.3))
        # gives player a visual clue as to range of nuke
        self.actorList.append(gnipactor.Circle2(pos, (radius, radius), (user.color, (0, 0, 0)), 2, galcon.FPS*2.0))

        # destroy some ships and change ownership to neutral
        inRange = gniputil.GetPlanetsInRadius(self.planets, pos, radius)
        for p in inRange:
            p.user = self.users[0] # make neutral
            p.ships -= p.ships * self.damagePercent # remove num of ships
        
    def paint(self, screen):
        returnVal = level.Level_.paint(self, screen)
        self.actorList.paint(screen)
        return(returnVal)



# patterned after function of same name in level.py
def Level(game,parent,opts,data=None):
    l_ = NukeMod(game,parent,opts,data)
    l = galcon.Level(l_)
    l_.state = l
    return l
