## Title: gnipmath.py
## Author: Scott O. Nelson (SirGnip)
## License: LGPL (c) 2007 Scott Nelson
## Description: math utilities

import math

def Lerp(x1, x2, u):
	'''linearly interpolate between two values (can be floats or vectors)'''
##	assert isinstance(x1, type(x2)), 'x1 and x2 must both be of the same type'
	return x1 + (x2 - x1)*u

def Clamp(val, min, max):
	if val < min:
		return min
	if val > max:
		return max
	return val

def ClampHi(val, hi):
	'''clamp value to hi if it is greater than hi'''
	if (val > hi):
		return hi
	return val

def ClampLo(val, lo):
	'''clamp value to lo if is less than lo'''
	if (val < lo):
		return lo
	return val



########################################
######################################## point/vector functions
########################################

def DistanceSquared(p1, p2):
	'''Returns the distance between the two points squared.  Often
	used as an optimization'''
	return((p2[0]-p1[0])**2 + (p2[1]-p1[1])**2)

def Distance(p1, p2):
	return(math.sqrt((p2[0]-p1[0])**2 + (p2[1]-p1[1])**2))

def PointLerp(x1, x2, u):
	'''linearly interpolate between two points (represented as tuples)'''
	x = Lerp(x1[0], x2[0], u)
	y = Lerp(x1[1], x2[1], u)
	return (x, y)
