## Title: gui.py
## Author: Phil Hassey
## License: LGPL (c) 2007 Phil Hassey
## Description: A simple GUI system.

import math
import galcon
Rect = galcon.Rect
Font = galcon.Font
class Event:
    def __init__(self,type_):
        self.type = type_

FOCUS,BLUR,ENTER,EXIT,CLICK = 'focus','blur','enter','exit','click'

def theme_x():
    img = galcon.Surface((14,14))
    img.fill((0,0,0,0))
    c = (0xFF,0xFF,0xFF)
    a,b,w = 2,11,3
    img.line(c,(a,a),(b,b),w)
    img.line(c,(a,b),(b,a),w)
    img.line(c,(a,a),(b,b),w)
    img.line(c,(a,b),(b,a),w)
    return img

theme = {
    ##############
    'label':{
        'font':Font(16),
        'color':(0xFF,0xFF,0xFF),
        },
        
    ##############
    'button':{
        'border':2,
        'border-color':(0x00,0xFF,0xFF),
        'padding':2,
        'margin':2,
        },
    'button:':{
        'background':(0x00,0x88,0x88),
        },
    'button:focus':{
        'background':(0x00,0xAA,0xAA),
        },
    'button:hover':{
        'background':(0x00,0xAA,0xAA),
        },
    'button:down':{
        'background':(0x00,0xFF,0xFF),
        },
    'button label':{
        'font':Font(16),
        'color':(0x00,0xFF,0xFF),
        },
    
    ##############
    'input':{
        'border':2,
        'border-color':(0x00,0xFF,0xFF),
        'padding':2,
        'margin':2,
        'background':(0x00,0x88,0x88),
        'width':120,
        'font':Font(16),
        'color':(0xFF,0xFF,0xFF),
        },
    'input:':{
        'background':(0x00,0x88,0x88),
        },
    'input:focus':{
        'background':(0x00,0xAA,0xAA),
        },
        
    ##############
    'checkbox':{
        'border':2,
        'border-color':(0x00,0xFF,0xFF),
        'margin':2,
        'padding':2,
        'width':14,
        'height':14,
        'icon':theme_x(),
        },
    'checkbox:':{
        'background':(0x00,0x88,0x88),
        },
    'checkbox:hover':{
        'background':(0x00,0xAA,0xAA),
        },
    'checkbox.off':{
        #'background':(0x00,0x88,0x88),
        },
    'checkbox.on':{
        #'background':(0x55,0xFF,0xFF),
        },
        
    ##############
    'radio':{
        'border':2,
        'border-color':(0x00,0xFF,0xFF),
        'margin':2,
        'padding':2,
        'width':14,
        'height':14,
        'icon':theme_x(),
        },
    'radio:':{
        'background':(0x00,0x88,0x88),
        },
    'radio:hover':{
        'background':(0x00,0xAA,0xAA),
        },
    'radio.off':{
        #'background':(0x00,0x88,0x88),
        },
    'radio.on':{
        #'background':(0x55,0xFF,0xFF),
        },
        
    ##############
    'slider':{
        'border':2,
        'border-color':(0x00,0xFF,0xFF),
        'margin':2,
        'color':(0xFF,0xFF,0xFF),
        'bar':(0x00,0xFF,0xFF),
        'font':Font(14),
        'width':120,
        'height':16,
        'size':32,
        },
    'slider:':{
        'background':(0x00,0x88,0x88),
        },
    'slider:hover':{
        'background':(0x00,0xAA,0xAA),
        },
    'slider:focus':{
        'background':(0x00,0xAA,0xAA),
        },
        
    }
    
def theme_resize(w,m):
    def func(width=None,height=None):
        s = w.style.get
        top,right,bottom,left = 0,0,0,0
        left += s('padding-left',s('padding',0))
        left += s('border-left',s('border',0))
        left += s('margin-left',s('margin',0))
        top += s('padding-top',s('padding',0))
        top += s('border-top',s('border',0))
        top += s('margin-top',s('margin',0))
        right += s('padding-right',s('padding',0))
        right += s('border-right',s('border',0))
        right += s('margin-right',s('margin',0))
        bottom += s('padding-bottom',s('padding',0))
        bottom += s('border-bottom',s('border',0))
        bottom += s('margin-bottom',s('margin',0))
        #w.s_top,w.s_right,w.s_bottom,w.s_left = top,right,bottom,left
        
        # get the widget to give its w & h
        if width != None and height != None:
            ww,hh = m(width-(left+right),height-(top+bottom))
        else:
            ww,hh = m(width,height)
        if width == None: width = left+ww+right
        if height == None: height = top+hh+bottom
        
        align = s('align',0)
        diff = width - (left + ww + right)
        inc = diff*(align+1)/2
        xx = left + inc
        
        align = s('valign',0)
        diff = height - (top + hh + bottom)
        inc = diff*(align+1)/2
        yy = top + inc
        
        w.t_rect = Rect(xx,yy,ww,hh)
        return width,height
    
    return func

def theme_paint(w,m):
    def func(screen):
        theme_style(w)
        
        s = w.style.get
        
        # calculate margin box
        r = w.rect
        x1 = 0 + s('margin-left',s('margin',0))
        y1 = 0 + s('margin-top',s('margin',0))
        x2 = r.w - s('margin-right',s('margin',0))
        y2 = r.h - s('margin-bottom',s('margin',0))
        
        # draw background
        c = s('background',None)
        if isinstance(c,tuple):
            screen.fill(c,(x1,y1,x2-x1,y2-y1))
        elif c != None:
            #0/0 use area parameter in blit maybe?
            screen.blit(c.subsurface(0,0,x2-x1,y2-y1),(x1,y1))
            
        # draw border box
        c = s('border-color',None)
        if c != None:
            size = s('border-left',s('border',0))
            if size: screen.fill(c,Rect(x1,y1,size,y2-y1))
            size = s('border-right',s('border',0))
            if size: screen.fill(c,Rect(x2-size,y1,size,y2-y1))
            size = s('border-top',s('border',0))
            if size: screen.fill(c,Rect(x1,y1,x2-x1,size))
            size = s('border-bottom',s('border',0))
            if size: screen.fill(c,Rect(x1,y2-size,x2-x1,size))
            
        tmp = w.rect
        w.rect = w.t_rect
        m(screen.subsurface(w.rect))
        w.rect = tmp
    return func
        

def theme_event(w,m):
    def func(e):
        if e.type is MOUSEBUTTONDOWN:
            w.t_state = True
        elif e.type is MOUSEBUTTONUP:
            w.t_state = False
        elif e.type is ENTER:
            w.t_hover = True
        elif e.type is EXIT:
            w.t_hover = False
        elif e.type is FOCUS:
            w.t_focus = True
        elif e.type is BLUR:
            w.t_focus = False
        
        e2 = e
        if e.type in (MOUSEBUTTONDOWN, MOUSEBUTTONUP):
            e2 = Event(e.type)
            e2.pos = e.pos[0]-w.t_rect.x,e.pos[1]-w.t_rect.y
            e2.button = e.button
        if e.type in (MOUSEMOTION,):
            e2 = Event(e.type)
            e2.pos = e.pos[0]-w.t_rect.x,e.pos[1]-w.t_rect.y
            e2.buttons = e.buttons
        tmp = w.rect
        w.rect = w.t_rect
        m(e2)
        w.rect = tmp
    return func

def theme_style(w):
    w.pseudo = ['']
    if w.t_hover: w.pseudo.append('hover')
    if w.t_focus: w.pseudo.append('focus')
    if w.t_state and w.t_hover and w.t_focus: w.pseudo.append('down')
    w.pseudo = tuple(w.pseudo)
    
    if w.t_element == w.element and w.t_klass == w.klass and w.t_pseudo == w.pseudo: return
    w.t_element,w.t_klass,w.t_pseudo = w.element,w.klass,w.pseudo
    
    #if w.element == 'checkbox': print w.element,w.klass,w.pseudo
    
    key = '%s'%(w.element)
    if key in theme: add_style(w.style,theme[key])
    for pseudo in w.pseudo:
        key = '%s:%s'%(w.element,pseudo)
        #if w.element == 'button': print key
        if key in theme: set_style(w.style,theme[key])
    if w.klass != None:
        key = '%s.%s'%(w.element,w.klass)
        if key in theme: set_style(w.style,theme[key])
        for pseudo in w.pseudo:
            key = '%s.%s:%s'%(w.element,w.klass,pseudo)
            if key in theme: set_style(w.style,theme[key])
    set_style(w.t_style,w.style)
    

def theme_deco(w):
    w.t_state = False
    w.t_hover = False
    w.t_focus = False
    w.resize = theme_resize(w,w.resize)
    w.paint = theme_paint(w,w.paint)
    w.event = theme_event(w,w.event)
    w.t_element = None
    w.t_klass = None
    w.t_pseudo = None
    w.t_style = {}
    theme_style(w)

        
# only to be called by Widget.__init__
def get_style(theme_style,**params):
    style = theme_style.copy()
    for k in ['width','height','background','align','valign']:
        if k in params: style[k] = params[k] 
    #style.setdefault('width',params.get('width',None))
    #style.setdefault('height',params.get('height',None))
    add_style(style,params.get('style',{}))
    return style
# this can be added onto by anyone ..
def add_style(style1,style2):
    for k,v in style2.items():
        if k not in style1: style1[k] = v
# or set the style more forcefully ..
def set_style(style1,style2):
    for k,v in style2.items():
        style1[k] = v

class Widget:
    def __init__(self,**params):
        if 'name' in params:
            self.name = params['name']
            self.form = Globals['form']
            self.form[self.name] = self
        self.value = None
        if 'value' in params:
            self.value = params['value']
        
        self.style = get_style({},**params)
        self.element = params.get('element',None)
        self.klass = params.get('klass',None)
        self.pseudo = None
        theme_deco(self)
    
    def event(self,e):
        pass
    def paint(self,screen):
        pass
    def resize(self,width=None,height=None):
        return self.style['width'],self.style['height']
    
    def refresh(self): # call this if you change some .value or whatever
        pass
    
    
Globals = {}
class Form(dict):
    def __init__(self):
        Globals['form'] = self

class Group(Widget):
    def __init__(self,name,**params):
        self.widgets = []
        params['name'] = name
        Widget.__init__(self,**params)
    def refresh(self):
        for w in self.widgets:
            w.refresh()

class Label(Widget):
    def __init__(self,value,**params):
        params['value'] = value
        params.setdefault('element','label')
        Widget.__init__(self,**params)
        self.refresh()
    def refresh(self):
        self.surf = self.style['font'].render(self.value,1,self.style['color'])
    def resize(self,width=None,height=None):
        return self.surf.get_width(),self.surf.get_height()
    def paint(self,screen):
        screen.blit(self.surf,(0,0))
        
    
class Spacer(Widget):
    def __init__(self,width,height,**params):
        params['width'] = width
        params['height'] = height
        Widget.__init__(self,**params)
    
class Image(Widget):
    def __init__(self,surf,**params):
        Widget.__init__(self,**params)
        params.setdefault('width',surf.get_width())
        params.setdefault('height',surf.get_height())
        self.surf = surf.scale((params['width'],params['height']))
    def paint(self,screen):
        screen.blit(self.surf,(0,0))
    def resize(self,width=None,height=None):
        return self.surf.get_width(),self.surf.get_height()
    
class Container(Widget):
    def __init__(self,**params):
        Widget.__init__(self,**params)
        self.widgets = []
        self.focus = None
        self.hover = None
    def paint(self,screen):
        for w in self.widgets:
            w.paint(screen.subsurface(w.rect))
    def add(self,w,pos):
        self.widgets.append(w)
        w.rect = Rect(pos[0],pos[1],0,0)
    def event(self,e):
        
        # handle focus / blur events
        prev,cur = self.focus,self.focus
        if e.type is MOUSEBUTTONDOWN:
            cur = None
            for w in self.widgets:
                if w.rect.collidepoint(e.pos): cur = w
        if e.type is BLUR: cur = None
        if prev != None and cur != prev: prev.event(Event(BLUR))
        if cur != None and cur != prev: cur.event(Event(FOCUS))
        self.focus = cur
            
        # handle enter / exit events
        prev,cur = self.hover,self.hover
        if e.type is MOUSEMOTION:
            cur = None
            for w in self.widgets:
                if w.rect.collidepoint(e.pos): cur = w
        if e.type is EXIT: cur = None
        if prev != None and cur != prev: prev.event(Event(EXIT))
        if cur != None and cur != prev: cur.event(Event(ENTER))
        self.hover = cur
        
        # if we're hovering, do that ...
        if self.hover != None and e.type is MOUSEMOTION:
            w = self.hover
            e2 = Event(e.type)
            e2.pos = e.pos[0]-w.rect.x,e.pos[1]-w.rect.y
            e2.buttons = e.buttons
            w.event(e2)
            
        # otherwise .. do whatever :)
        if self.focus != None:
            w = self.focus
            e2 = e
            if e.type in (MOUSEBUTTONDOWN, MOUSEBUTTONUP):
                e2 = Event(e.type)
                e2.pos = e.pos[0]-w.rect.x,e.pos[1]-w.rect.y
                e2.button = e.button
            if e.type in (MOUSEMOTION,):
                e2 = Event(e.type)
                e2.pos = e.pos[0]-w.rect.x,e.pos[1]-w.rect.y
                e2.buttons = e.buttons
            w.event(e2)
            
            if e.type is MOUSEBUTTONUP and self.focus == self.hover:
                w.event(Event(CLICK))
    
    def resize(self,width=None,height=None):
        ww,hh = 0,0
        for w in self.widgets:
            w.rect.w,w.rect.h = w.resize()
            ww = max(ww,w.rect.x+w.rect.w)
            hh = max(hh,w.rect.y+w.rect.h)
        return ww,hh
    
    
class Table(Container):
    def __init__(self,**params):
        Container.__init__(self,**params)
        self.row = -1
        self.col = 0
        self.grid = {}
        
    def tr(self):
        self.row,self.col = self.row+1,0
        
    def td(self,w,colspan=1,rowspan=1):
        grid = self.grid
        self.row = max(0,self.row) # force the initial tr()
        col,row = self.col,self.row
        while grid.get((col,row),None) != None: col += 1
        w.cell = (col,row)
        w.colspan = colspan
        w.rowspan = rowspan
        for y in xrange(w.cell[1],w.cell[1]+w.rowspan):
            for x in xrange(w.cell[0],w.cell[0]+w.colspan):
                grid[(x,y)] = w
        self.col = col + colspan
        self.add(w,(0,0))
        
    def resize(self,width=None,height=None):
        # figure out how many rows, cols we have & resize widgets
        cols,rows = 0,0
        for w in self.widgets:
            cols = max(cols,w.cell[0]+w.colspan)
            rows = max(rows,w.cell[1]+w.rowspan)
            w.rect.w,w.rect.h = w.resize()
        cols = [0 for n in xrange(0,cols)]
        rows = [0 for n in xrange(0,rows)]
        
        # size out our cols
        def sort_by_colspan(a,b):
            return a.colspan-b.colspan
        self.widgets.sort(sort_by_colspan)
        for w in self.widgets:
            v = sum([cols[x] for x in xrange(w.cell[0],w.cell[0]+w.colspan)])
            if v >= w.rect.w: continue
            diff = w.rect.w - v
            inc = math.ceil(float(diff) / float(w.colspan))
            for x in xrange(w.cell[0],w.cell[0]+w.colspan): cols[x] += inc
            
        # size out our rows
        def sort_by_rowspan(a,b):
            return a.rowspan-b.rowspan
        self.widgets.sort(sort_by_rowspan)
        for w in self.widgets:
            v = sum([rows[x] for x in xrange(w.cell[1],w.cell[1]+w.rowspan)])
            if v >= w.rect.h: continue
            diff = w.rect.h - v
            inc = math.ceil(float(diff) / float(w.rowspan))
            for x in xrange(w.cell[1],w.cell[1]+w.rowspan): rows[x] += inc
            
        # calc positions
        ww = 0
        cols_pos = []
        for n in xrange(0,len(cols)):
            cols_pos.append(ww)
            ww += cols[n]
        
        hh = 0
        rows_pos = []
        for n in xrange(0,len(rows)):
            rows_pos.append(hh)
            hh += rows[n]
            
        # move widgets into position
        for w in self.widgets:
            w.rect.x,w.rect.y = cols_pos[w.cell[0]],rows_pos[w.cell[1]]
            w2 = sum([cols[x] for x in xrange(w.cell[0],w.cell[0]+w.colspan)])
            h2 = sum([rows[x] for x in xrange(w.cell[1],w.cell[1]+w.rowspan)])
            w.rect.w,w.rect.h = w.resize(w2,h2)
            
        return ww,hh

        
class Button(Widget):
    def __init__(self,widget,**params):
        params.setdefault('element','button')
        Widget.__init__(self,**params)
        if isinstance(widget,str):
            widget = Label(widget)
        self.widget = widget
        
    def resize(self,width=None,height=None):
        return self.widget.resize(width,height)
    
    def paint(self,screen):
        self.widget.rect = self.rect
        self.widget.paint(screen)
        
    def event(self,e):
        if e.type is CLICK:
            self.onclick()
            
    def onclick(self):
        pass

class Input(Widget):
    def __init__(self,name,**params):
        params['name'] = name
        params.setdefault('value','')
        params.setdefault('element','input')
        Widget.__init__(self,**params)
        self.refresh()
        
    def event(self,e):
        if e.type is KEYDOWN and e.key == K_BACKSPACE:
            self.value = self.value[:-1]
            self.refresh()
        elif e.type is KEYDOWN:
            c = galcon.key_get_char(e.unicode)
            self.value += c
            self.refresh()

    def refresh(self):
        self.surf = self.style['font'].render(self.value+'|',1,self.style['color'])

    def paint(self,screen):
        img = self.surf
        x = 0
        ww = img.get_width()
        if ww > self.rect.w:
            x = self.rect.w-ww
        screen.blit(img,(x,0))
        
    def resize(self,width=None,height=None):
        if width == None: width = self.style['width']
        return width,self.style['font'].get_height()
            
class Checkbox(Widget):
    def __init__(self,name,**params):
        params['name'] = name
        params.setdefault('value',False)
        params.setdefault('element','checkbox')
        Widget.__init__(self,**params)
        self.refresh()
        
    def event(self,e):
        if e.type is CLICK:
            self.click()
            
    def click(self):
        self.value = not self.value
        self.refresh()
        
    def refresh(self):
        self.klass = 'off'
        if self.value: self.klass = 'on'
        
    def paint(self,screen):
        if self.klass == 'on':
            screen.blit(self.style['icon'],(0,0))

class Radio(Widget):
    def __init__(self,group,value,**params):
        params['value'] = value
        params.setdefault('element','radio')
        self.group = group
        self.group.widgets.append(self)
        Widget.__init__(self,**params)
        self.refresh()
        
    def event(self,e):
        if e.type is CLICK:
            self.click()
            
    def refresh(self):
        self.klass = 'off'
        if self.value == self.group.value: self.klass = 'on'
        
    def click(self):
        self.group.value = self.value
        self.group.refresh()
        
    def paint(self,screen):
        if self.klass == 'on':
            screen.blit(self.style['icon'],(0,0))

class Slider(Widget):
    def __init__(self,name,min_,max_,**params):
        params['name'] = name
        params.setdefault('value',None)
        params.setdefault('element','slider')
        Widget.__init__(self,**params)
        self.max_ = max_
        self.min_ = min_
        if self.value == None: self.value = (self.min_+self.max_)/2
        self.refresh()
        
    def event(self,e):
        if e.type is MOUSEMOTION:
            if not self.t_focus: return
            if 1 not in e.buttons: return
            size = self.style['size']
            x = e.pos[0]
            v = float(x-size/2) * float(self.max_-self.min_) / float(self.rect.w-size) + self.min_
            v = int(round(max(self.min_,min(self.max_,v))))
            self.value = v
            self.refresh()
            
    def refresh(self):
        self.surf = self.style['font'].render(str(self.value),1,self.style['color'])
            
    def paint(self,screen):
        size = self.style['size']
        x = (self.value-self.min_) * (self.rect.w-size) / (self.max_-self.min_) + size/2
        r = Rect(x-size/2,0,size,self.rect.h)
        screen.fill(self.style['bar'],r)
        img = self.surf
        pos = (x-img.get_width()/2,self.rect.h/2-img.get_height()/2)
        screen.blit(img,pos)
        
    def resize(self,width=None,height=None):
        if width == None or height == None:
            return Widget.resize(self,width,height)
        return width,height
    
class Box(Container):
    def __init__(self,widget,**params):
        Container.__init__(self,**params)
        self.add(widget,(0,0))
        self.widget = widget
        
    def resize(self,width=None,height=None):
        s = self.style.get
        w = self.widget
        w.rect.w,w.rect.h = w.resize()
        
        width = max(width,w.rect.w,s('width',0))
        height = max(height,w.rect.h,s('height',0))
        
        dx = width-w.rect.w
        dy = height-w.rect.h
        w.rect.x = (s('align',0)+1)*dx/2
        w.rect.y = (s('valign',0)+1)*dy/2
        
        return width,height

import dialog
class App(galcon.State):
    def __init__(self,game,w,parent):
        self.game = game
        self.widget = w
        w.rect = Rect(0,0,SW,SH)
        self.next = None
        self.parent = parent
    def init(self):
        self.widget.resize()
    def event(self,e):
        if e.type is KEYDOWN and e.key == K_ESCAPE:
            return dialog.Prompt(self.game,'quit?',self.parent,self)
        self.widget.event(e)
    def paint(self,screen):
        self.widget.paint(screen)
    def goto(self,state):
        self.next = state
    def loop(self):
        r = None
        if self.next != None:
            r = self.next
            self.next = None
        return r
