## Title: dialog.py
## Author: Phil Hassey
## License: LGPL (c) 2007 Phil Hassey
## Description: Various handy dialogs.

import galcon

class Help(galcon.State):
    def __init__(self,game,lines,next):
        self.game,self.lines,self.next = game,lines,next
        
    def init(self):
        self.font = self.game.font
        self.bkgr = None
        
    def paint(self,screen):
        if not self.bkgr: self.bkgr = fade(screen)
        screen.blit(self.bkgr,(0,0))
        
        fnt = self.font
        c = (255,255,255)
        h = fnt.get_height()
        x,y = 0,(SH-h*len(self.lines))/2
        for text in self.lines:
            img = fnt.render(text,1,c)
            x = (galcon.SW-img.get_width())/2
            screen.blit(img,(x,y))
            y += h
            
    def event(self,e):
        if e.type is KEYDOWN or e.type is MOUSEBUTTONUP:
            return return_(self.next)

class FileDialog(galcon.State):
    def __init__(self,game,text,fname,yes,no):
        self.game,self.text,self.fname,self.yes,self.no = game,text,fname,yes,no
        
    def init(self):
        self.font = self.game.font
        self.bkgr = None
        
    def paint(self,screen):
        if not self.bkgr: self.bkgr = fade(screen)
        screen.blit(self.bkgr,(0,0))
        
        fnt = self.font
        c = (255,255,255)
        text = '%s %s_'%(self.text,self.fname)
        img = fnt.render(text,1,c)
        x = (galcon.SW-img.get_width())/2
        y = (galcon.SH-img.get_height())/2
        screen.blit(img,(x,y))
        
    def event(self,e):
        if e.type is galcon.KEYDOWN and e.key == galcon.K_RETURN:
            return return_(self.yes,self.fname)
        
        if e.type is galcon.KEYDOWN and e.key == galcon.K_BACKSPACE:
            self.fname = self.fname[:-1]
            self.repaint()
            return
        
        if e.type is galcon.KEYDOWN and e.key == galcon.K_ESCAPE:
            return return_(self.no)
        
        if e.type is KEYDOWN:
            self.fname += galcon.key_get_char(e.unicode)
            


class Choice(galcon.State):
    def __init__(self,game,text,items):
        self.game,self.text,self.items = game,text,items

    def init(self):
        self.font = self.game.font
        self.bkgr = None
        self.rects = []
        self.cur = 0
        
    def paint(self,screen):
        if not self.bkgr: self.bkgr = fade(screen)
        screen.blit(self.bkgr,(0,0))

        fnt = self.font
        c = (255,255,255)
        h = fnt.get_height()
        total = len(self.items)
        if self.text != None: total += 1
        y = (SH-h*total)/2
        
        if self.text != None:
            img = fnt.render(self.text,1,c)
            x = (galcon.SW-img.get_width())/2
            screen.blit(img,(x,y))
            y += h
        
        n = 0
        self.rects = []
        for text,key,value in self.items:
            c = (0x88,0x88,0x88)
            if n == self.cur:
                c = (0x88,0xff,0xff)
            img = fnt.render(text,1,c)
            x = (galcon.SW-img.get_width())/2
            screen.blit(img,(x,y))
            r = galcon.Rect(x,y,img.get_width(),img.get_height())
            self.rects.append((r,n))
            y += h
            n += 1
         
    def event(self,e):
        if e.type is galcon.KEYDOWN and e.key == K_UP:
            self.cur = (self.cur-1+len(self.items))%len(self.items)
            self.repaint()
            
        if e.type is KEYDOWN and e.key == K_DOWN:
            self.cur = (self.cur+1+len(self.items))%len(self.items)
            self.repaint()
            
        if e.type is MOUSEMOTION:
            for r,n in self.rects:
                if r.collidepoint(e.pos):
                    self.cur = n
                    self.repaint()
                    
        if e.type is MOUSEBUTTONUP or (e.type is KEYDOWN and e.key == K_RETURN):
            return return_(self.items[self.cur][2])
        
        for text,key,value in self.items:
            if e.type is KEYDOWN and e.key == key:
                return return_(value)
            
def Prompt(game,text,yes,no):
    return Choice(game,text,[
        ('yes',K_y,yes),
        ('no',K_n,no),
        ])

class Pause(galcon.State):
    def __init__(self,game,text,next):
        self.game,self.text,self.next = game,text,next

    def init(self):
        self.font = self.game.font
        self.bkgr = None
        
    def paint(self,screen):
        if not self.bkgr: self.bkgr = fade(screen)
        screen.blit(self.bkgr,(0,0))

        fnt = self.font
        c = (255,255,255)
        img = fnt.render(self.text,1,c)
        x = (galcon.SW-img.get_width())/2
        y = (galcon.SH-img.get_height())/2
        screen.blit(img,(x,y))
         
    def event(self,e):
        if e.type is galcon.KEYDOWN or e.type is MOUSEBUTTONUP:
            return return_(self.next)

def fade(s):
    s = s.copy()
    i = galcon.Surface((s.get_width(),s.get_height()))
    i.fill((0,0,0,128))
    s.blit(i,(0,0))
    return s

def return_(v,*params):
    if isinstance(v,list) or isinstance(v,tuple):
        return v[0](*v[1:])
    if callable(v):
        return v(*params)
    return v

