## Title: mclassic.py
## Author: Phil Hassey, modified by ThirdParty
## License: Public Domain
## Description: The classic Galcon mission.

import galcon
import dialog

class Level(galcon.State):
	# the classic level requires
	# .copy - a copy of this level to retry
	# .timer - set to 0
	# .frame - set to 0
	# .parent - parent to return to 
	# .state - the state we're actually in ..
   
	def loop(self):
		for u in self.users.values():
			u.loop()
			
		for p in self.planets.values():
			if p.user.n == 0: continue
			p.ships += p.production / (galcon.FPS*50.0)
			
		livingteam,twosides,twofleets = 0,False,False
		for p in self.planets.values():
			if not isinstance(p.user,galcon.Neutral):
				if livingteam == 0:
					livingteam = p.user.team
				elif p.user.team != livingteam:
					twosides = True
		if twosides: twofleets = True
		else:
			for f in self.fleets.values():
				if livingteam == 0:
					livingteam = f.user.team
				elif f.user.team != livingteam:
					twofleets = True
		if twosides:
			# so long as everyone has planets, the timer stays at thirty seconds
			self.timer = FPS*30
		else:
			# when one side has no planets, the timer starts ticking down
			self.timer -= 1
			if (not twofleets) and (self.timer > FPS*2):
				# if one side is eliminated, lower the timer to two seconds if necessary
				self.timer = FPS*2
			# end the game when the timer reaches zero
			if self.timer < 1:
				if livingteam != self.player.team:
					if self.opts['symmetry']:
						return dialog.Choice(self.game,'Game Over',[
							('restart',K_r,self.reset),
							('new game',K_q,self.parent),
							('quit',None,False),
							])
					else:
						return dialog.Choice(self.game,'Game Over',[
							('restart',K_r,self.reset),
							('rotate and restart',None,self.resetandrotate),
							('new game',K_q,self.parent),
							('quit',None,False),
							])
				else:
					if self.opts['symmetry']:
						return dialog.Choice(self.game,'Good Job!',[
							('restart',K_r,self.reset),
							('new game',K_q,self.parent),
							('quit',None,False),
							])
					else:
						return dialog.Choice(self.game,'Good Job!',[
							('restart',K_r,self.reset),
							('rotate and restart',None,self.resetandrotate),
							('new game',K_q,self.parent),
							('quit',None,False),
							])
			
		self.frame += 1
	
	def fleet(self,user,from_,to,ships):
		if from_.user != user or ships <= 0: return
		f = galcon.Fleet()
		f.user = user
		f.from_ = from_
		f.to = to
		if ships > from_.ships: f.ships = from_.ships
		else: f.ships = ships
		from_.ships -= f.ships
		self.fleets.add(f)
		
	def redirect(self,user,from_,to):
		if from_.user != user: return
		from_.to = to
		
	def hit(self,fleet,ships,p):
		s = p.ships
		if p.user.team == fleet.user.team:
			p.ships += ships
		else:
			p.ships -= ships
			if p.ships < 0:
				p.ships = -p.ships
				p.user = fleet.user
		return p.ships-s
	
	def event(self,e):
		if e.type is KEYDOWN and e.key == K_ESCAPE:
			if self.opts['symmetry']:
				return dialog.Choice(self.game,'Game Paused',[
					('continue',K_c,self.state),
					('restart',K_r,self.reset),
					('new game',K_q,self.parent),
					('quit',None,False),
					])
			else:
				return dialog.Choice(self.game,'Game Paused',[
					('continue',K_c,self.state),
					('restart',K_r,self.reset),
					('rotate and restart',None,self.resetandrotate),
					('new game',K_q,self.parent),
					('quit',None,False),
					])
		 
	def paint(self,screen):
		if self.first:
			self.first = False
			return dialog.Pause(self.game,'get ready',self.state)
