import galcon
import random
rr = random.randrange
import math

import thirdbots
import colorscheme
import classic
import dialog
import mapfile

class Level_(classic.Level):
	def __init__(self,game,parent,handicap = 0,data = None):
		self.help = [
			'---------- Goliath ----------',
			'',
			'Our enemy\'s starting position is better than ours.',
			'We need to take it away from him.',
			'',
			'Tip:',
			'There is nothing special about your homeworld; do not',
			'try to defend it if other territory is more valuable.',
			'If an enemy has a better position than you, position your',
			'ships near his territory, wait for him to spend his ships',
			'on neutrals or to send them elsewhere (such as into',
			'your territory), and then attack quickly.'
			]
		self.game = game
		self.parent = parent
		self.handicap = handicap
		self.data = data
		
	def init(self):
		# set up the player
		u = galcon.Player()
		u.name = 'Player'
		u.color = colorscheme.human
		self.users.add(u)
		self.player = u

		# set up the bot
		u = thirdbots.Bot_Cobra(self,2.0 / (2 + self.handicap))
		u.name = 'Goliath'
		u.color = colorscheme.cobra
		self.users.add(u)
		
		if self.data == None:
			self.generate()
			self.data = mapfile.tostring(self)
		else:
			mapfile.fromstring(self,self.data)

		self.sync()
			
		# initialize all the users / bots
		for u in self.users.values():
			u.init()
			
		# misc
		self.timer,self.frame,self.first = 0,0,True
		
	def copy(self):
		return Level(self.game,self.parent,self.handicap,self.data)
	
	def generate(self):
		# a list of home planets
		homes = []
		
		# set up the player's home planet
		u = self.users[1]
		p = galcon.Planet()
		p.production = 100
		p.ships = 100
		p.user = u
		self.planets.add(p)
		homes.append(p)
		
		# set up the bot's home planet
		u = self.users[2]
		p = galcon.Planet()
		p.production = 200
		p.ships = 160 - 5 * self.handicap
		p.user = u
		self.planets.add(p)
		homes.append(p)
		
		# add 15 random planets
		for n in xrange(0,15):
			p = galcon.Planet()
			p.production = rr(0,100)
			p.ships = rr(0,50)
			p.user = self.users[0]
			self.planets.add(p)
		
		# set the scale - this formula works good
		self.options.scale = 1.0/((len(self.planets)/23.0)**0.5)
		
		# running this function syncs the data here with galcon
		# and gets valuable stuff like planet radius figured out, etc
		self.sync()
		
		# get some basic gist of the screen sizing ...
		sz = 16
		top = galcon.BORDER+sz
		left = galcon.PADDING+sz

		# randomize the position of the planets
		for p in self.planets.values():
			p.pos = rr(left+p.r,galcon.SW-p.r-left),rr(top+p.r,galcon.SH-p.r-top)

		# place the home planets in a circular formation
		SW,SH = galcon.SW,galcon.SH
		random.shuffle(homes)
		a = rr(0,360)
		inc = 360/len(homes)
		for p in homes:
			x = SW/2+(SH/2)*math.cos(a*math.pi*2.0/360.0)
			y = SH/2+(SH/2)*math.sin(a*math.pi*2.0/360.0)
			p.pos = x,y
			a += inc


def Level(game,parent,handicap=0,data=None):
	l_ = Level_(game,parent,handicap,data)
	l = galcon.Level(l_)
	l_.state = l
	return l
	