import galcon
import random
rr = random.randrange
import math

import thirdbots
import colorscheme
import classic
import dialog
import mapfile

class Level_(classic.Level):
	def __init__(self,game,parent,handicap = 0,data = None):
		self.game = game
		self.parent = parent
		self.handicap = handicap
		self.data = data
		
	def init(self):
		self.help = [
			'---------- Pairs ----------',
			'',
			'It is a two versus two situation.',
			'The enemy is slightly more powerful than we are,',
			'but with good teamwork, we can win.',
			'',
			'Tip:',
			'Pay attention to what your partner is doing,',
			'assisting him or reinforcing as necessary.'
			]

		# create some teams
		t = galcon.Team()
		t.name = 'Good Guys'
		t.color = colorscheme.human
		self.teams.add(t)
		t = galcon.Team()
		t.name = 'Bad Guys'
		t.color = colorscheme.libra
		self.teams.add(t)
		
		# set up the player
		u = galcon.Player()
		u.name = 'Player'
		u.color = colorscheme.human
		self.users.add(u)
		self.player = u
		u.team = self.teams[1]

		# set up the bots

		u = thirdbots.Bot_Lass(self)
		u.name = 'Friendly'
		u.color = colorscheme.lass
		self.users.add(u)
		u.team = self.teams[1]
		
		u = thirdbots.Bot_Libra(self, 2.0 / (2 + self.handicap))
		u.name = 'Angry'
		u.color = colorscheme.libra
		self.users.add(u)
		u.team = self.teams[2]
		
		u = thirdbots.Bot_Cobra(self, 2.0 / (2 + self.handicap))
		u.name = 'Greedy'
		u.color = colorscheme.cobra
		self.users.add(u)
		u.team = self.teams[2]
		
		if self.data == None:
			self.generate()
			self.data = mapfile.tostring(self)
		else:
			mapfile.fromstring(self,self.data)
			
		self.sync()
		# initialize all the users / bots
		for u in self.users.values():
			u.init()
			
		# misc
		self.timer,self.frame,self.first = 0,0,True
		
	def copy(self):
		return Level(self.game,self.parent,self.handicap,self.data)
	
	def generate(self):
		# a list of home planets
		homes = []
		
		for u in self.users.values():
			if u.n == 0: continue
			p = galcon.Planet()
			p.production = 100
			if u.team == self.teams[1]: p.ships = 100
			else: p.ships = 128 - self.handicap*4
			p.user = u
			self.planets.add(p)
			homes.append(p)
		
		# add 41 random planets
		for n in xrange(0,41):
			p = galcon.Planet()
			p.production = rr(0,100)
			p.ships = rr(0,50)
			p.user = self.users[0]
			self.planets.add(p)
		
		# set the scale - this formula works good
		self.options.scale = 1.0/((len(self.planets)/23.0)**0.5)
		
		# running this function syncs the data here with galcon
		# and gets valuable stuff like planet radius figured out, etc
		self.sync()
		
		# get some basic gist of the screen sizing ...
		sz = 16
		top = galcon.BORDER+sz
		left = galcon.PADDING+sz

		# randomize the position of the planets
		for p in self.planets.values():
			p.pos = rr(left+p.r,galcon.SW-p.r-left),rr(top+p.r,galcon.SH-p.r-top)

		# place the home planets in a circular formation
		SW,SH = galcon.SW,galcon.SH
		random.shuffle(homes)
		a = rr(0,360)
		inc = 360/len(homes)
		for p in homes:
			x = SW/2+(SH/2)*math.cos(a*math.pi*2.0/360.0)
			y = SH/2+(SH/2)*math.sin(a*math.pi*2.0/360.0)
			p.pos = x,y
			a += inc

def Level(game,parent,handicap=0,data=None):
	l_ = Level_(game,parent,handicap,data)
	l = galcon.Level(l_)
	l_.state = l
	return l
	