import galcon
import random
rr = random.randrange
import math

import thirdbots
import colorscheme
import classic
import dialog
import mapfile

class Level_(classic.Level):
	def __init__(self,game,parent,handicap = 0,data = None):
		self.help = [
			'---------- Rebellion ----------',
			'',
			'This is as even a situation as I have ever seen,',
			'but we have a problem.  Our planets are not content',
			'with our rule, and are trying to rebel.',
			'The rebels do not have many forces, but',
			'if they establish themselves it will be a disaster.',
			'',
			'Tip:',
			'When a third party is trying to (re)establish',
			'himself, keep a garrison on all of your',
			'planets and pay attention to your rear.'
			]
		self.game = game
		self.parent = parent
		self.handicap = handicap
		self.data = data
		self.rebels = 0
		
	def init(self):
		# set up the player
		u = galcon.Player()
		u.name = 'Player'
		u.color = colorscheme.human
		self.users.add(u)
		self.player = u

		# set up the main enemy
		u = thirdbots.Bot_Libra(self,2.0 / (2 + self.handicap))
		u.name = 'Nemesis'
		u.color = colorscheme.libra
		self.users.add(u)

		# set up the rebels
		u = thirdbots.Bot_Vendetta(self)
		u.name = 'Rebels'
		u.color = colorscheme.vendetta
		self.users.add(u)
		self.rebel = u
		
		if self.data == None:
			self.generate()
			self.data = mapfile.tostring(self)
		else:
			mapfile.fromstring(self,self.data)

		self.sync()
			
		# initialize all the users / bots
		for u in self.users.values():
			u.init()
			
		# misc
		self.timer,self.frame,self.first = 0,0,True
		
	def copy(self):
		return Level(self.game,self.parent,self.handicap,self.data)
	
	def loop(self):
		for u in self.users.values():
			u.loop()
			
		for p in self.planets.values():
			if p.user.n == 0: continue
			p.ships += p.production / (galcon.FPS*50.0)

		# rebels code
		self.rebels += (4 * (10-self.handicap)) / (galcon.FPS*50.0) # we have invisible production
		playerplanets = []
		for p in self.planets.values():
			if p.user == self.player: playerplanets.append(p) # we're sending our ships invisibly flying around the player's territory
		if len(playerplanets) > 0:
			p = random.choice(playerplanets)
			needed = p.ships+1 # if our fleet can conquer a planet...
			needed += p.production * 5 / 50 # in the time it would take to get there...
			for f in self.fleets.values(): # even counting reinforcements...
				if f.to != p or f.user == self.rebel: continue
				needed += f.ships
			if needed < self.rebels: # ... then try to land
				f = galcon.Fleet()
				f.user = self.rebel
				f.from_ = p
				f.to = p
				f.ships = self.rebels
				self.fleets.add(f)
				self.rebels = 0
			
		livingteam,twosides,twofleets = 0,False,False
		for p in self.planets.values():
			if not isinstance(p.user,galcon.Neutral):
				if livingteam == 0:
					livingteam = p.user.team
				elif p.user.team != livingteam:
					twosides = True
		if twosides: twofleets = True
		else:
			for f in self.fleets.values():
				if livingteam == 0:
					livingteam = f.user.team
				elif f.user.team != livingteam:
					twofleets = True
		# self.player.team
		if twosides:
			# so long as everyone has planets, the timer stays at thirty seconds
			self.timer = FPS*30
		else:
			# when one side has no planets, the timer starts ticking down
			self.timer -= 1
			if (not twofleets) and (self.timer > FPS*2):
				# if one side is eliminated, lower the timer to two seconds if necessary
				self.timer = FPS*2
			# end the game when the timer reaches zero
			if self.timer < 1:
				if livingteam == self.player.team:
					return dialog.Pause(self.game,'good job!',self.parent)
				else:
					return dialog.Prompt(self.game,'try again?',self.copy,self.parent)
			
		self.frame += 1
	
	def generate(self):
		# get some basic gist of the screen sizing ...
		sz = 16
		SW,SH = galcon.SW,galcon.SH
		top = galcon.BORDER+sz
		left = galcon.PADDING+sz
		bottom = SH-top
		right = SW-left

		# a list of home planets
		homes = []
		
		# set up player's home planet
		u = self.users[1]
		p = galcon.Planet()
		p.production = 100
		p.ships = 100
		p.user = u
		self.planets.add(p)
		homes.append(p)
		
		# set up nemesis's home planet
		u = self.users[2]
		p = galcon.Planet()
		p.production = 100
		p.ships = 100
		p.user = u
		self.planets.add(p)
		homes.append(p)
		
		# add 18 pairs of random planets
		planetpairs = []
		for n in xrange(0,18):
			p = galcon.Planet()
			p.production = rr(-14,114)
			p.ships = rr(-6,56)
			p.user = self.users[0]
			q = galcon.Planet()
			q.production = p.production
			q.ships = p.ships
			q.user = self.users[0]

			p.production = rr(max(0,p.production-15),min(100,p.production+15))
			q.production = rr(max(0,q.production-15),min(100,q.production+15))
			p.ships = rr(max(0,p.ships-7),min(50,p.ships+7))
			q.ships = rr(max(0,q.ships-7),min(50,q.ships+7))
			self.planets.add(p)
			self.planets.add(q)
			planetpairs.append([p,q])
		
		# set the scale - this formula works good
		self.options.scale = 1.0/((len(self.planets)/23.0)**0.5)
		
		# running this function syncs the data here with galcon
		# and gets valuable stuff like planet radius figured out, etc
		self.sync()

		# randomize the position of the planet pairs
		for p,q in planetpairs:
			x,y = rr(left,right),rr(top,bottom)
			p.pos = rr(max(x-SW/8,left+p.r),min(x+SW/8,right-p.r)),rr(max(y-SH/8,top+p.r),min(y+SH/8,bottom-p.r))
			x = SW-x; y = SH-y
			q.pos = rr(max(x-SW/8,left+q.r),min(x+SW/8,right-q.r)),rr(max(y-SH/8,top+q.r),min(y+SH/8,bottom-q.r))

		# place the home planets in a circular formation
		random.shuffle(homes)
		a = rr(0,360)
		inc = 360/len(homes)
		for p in homes:
			x = SW/2+(SH/2)*math.cos(a*math.pi*2.0/360.0)
			y = SH/2+(SH/2)*math.sin(a*math.pi*2.0/360.0)
			p.pos = x,y
			a += inc

def Level(game,parent,handicap=0,data=None):
	l_ = Level_(game,parent,handicap,data)
	l = galcon.Level(l_)
	l_.state = l
	return l
	