import galcon
import random
rr = random.randrange
import math

import thirdbots
import colorscheme
import classic
import dialog
import mapfile

class Level_(classic.Level):
	def __init__(self,game,parent,handicap = 0,data = None,timeallowed = -1):
		self.help = [
			'---------- Vacuum ----------',
			'',
			'There is no enemy here, but we are on a deadline.',
			'Capture all planets as quickly as possible.',
			'',
			'Tip:',
			'Large and cheap planets are better initial',
			'targets than small and expensive ones.',
			'It is also good to focus on nearby planets;',
			'try to minimize your ships\' flight time.'
			]
		self.game = game
		self.parent = parent
		self.handicap = handicap
		self.data = data
		self.timeallowed = timeallowed
	def init(self):
		# set up the player
		u = galcon.Player()
		u.name = 'Player'
		u.color = colorscheme.human
		self.users.add(u)
		self.player = u
		
		if self.data == None:
			self.generate()
			self.data = mapfile.tostring(self)
			self.timeallowed = -1
		else:
			mapfile.fromstring(self,self.data)

		self.sync()
			
		# initialize all the users / bots
		for u in self.users.values():
			u.init()
			
		# misc
		self.timer,self.frame,self.first = 0,0,True
		
		# compute time needed to conquer

		if self.timeallowed < 0:
			kActspersecond = 2

			kMinfleet = 1
			kMaxunscaleddist = 0
			for p in self.planets.values():
				for b in self.planets.values():
					dist = math.hypot(p.pos[0]-b.pos[0],p.pos[1]-b.pos[1])
					if dist > kMaxunscaleddist: kMaxunscaleddist = dist

			self.timeallowed = 0
			planetlist = [] # stored as [ships needed, time before capture, production, location]
			for p in self.planets.values():
				if p.user == self.player: planetlist.append([-p.ships, 0, p.production, p.pos])
				else: planetlist.append([p.ships+1, 0, p.production, p.pos])

			while True:
				# simulate production and check for wins
				self.timeallowed += FPS / kActspersecond
				won = True
				for p in planetlist:
					if p[1] <= 0 and p[0] <= 0:
						p[0] -= p[2] / (kActspersecond * 50.0)
						plan = p
					else:
						if p[1] > 0: p[1] -= FPS/kActspersecond
						won = False
				if won: break

				# CONDUCT CENSUS

				myTargets = [] # target entries have form [mydist, planetlist entry]
				myPlanets = [] # planet etnries have form [temp, planetlist entry]
				myForces = 0
				myProd = 0

				enplanets = []

				for p in planetlist:
					if p[1] <= 0 and p[0] <= 0:
						myProd += p[2]
						myForces += 0-p[0]
						if 0-p[0] >= kMinfleet:
							myPlanets.append([0,p])
					elif p[0] > 0: enplanets.append(p)
				if len(myPlanets) == 0: continue

				for p in enplanets:
					mydist = kMaxunscaleddist
					for e in myPlanets:
						dist = math.hypot(p[3][0]-e[1][3][0],p[3][1]-e[1][3][1])
						if dist < mydist: mydist = dist
					mydist /= self.options.scale

					myTargets.append([mydist,p])

				if len(myTargets) == 0: continue

		
				# CHOOSE TARGET

				value = -10000
				target = None

				for p in myTargets:
					t = p[0] # how long would it take to get there?
					t += p[1][0] * 50 * FPS / myProd # measure the cost in terms of how long it takes us to build that cost
					v = p[1][2] * 1000.0 / t # find the best production-to-cost ratio

					if target and v < value: continue
					value,target = v,p

				if target == None: continue

				# ATTACK TARGET

				for p in myPlanets: p[0] = math.hypot(p[1][3][0]-target[1][3][0],p[1][3][1]-target[1][3][1]) / (self.options.scale)
				myPlanets.sort()

				reqforces = target[1][0]
				countedprod = 0
				counteddist = 0
		
				forcenum = len(myPlanets)
				i = 0
				while i < forcenum and reqforces > 0:
					p = myPlanets[i]
					i += 1
					if 0-p[1][0] >= reqforces:
						shipq = reqforces
						p[1][0] += shipq
						target[1][0] -= shipq
						if target[1][1] < p[0] * self.options.scale: target[1][1] = p[0] * self.options.scale

						reqforces = 0
						break
					else:
						shipq = 0-p[1][0]
						p[1][0] += shipq
						target[1][0] -= shipq
						if target[1][1] < p[0] * self.options.scale: target[1][1] = p[0] * self.options.scale

						reqforces -= shipq
						reqforces -= (p[0] - counteddist) * countedprod / (FPS * 50)
						counteddist = p[0]
						countedprod += p[1][2]

			self.timeallowed += 2 * FPS * (self.handicap+2)
	
	def loop(self):
		for u in self.users.values():
			u.loop()
		
		done = True
		for p in self.planets.values():
			if p.user.n == 0: done = False
			else: p.ships += p.production / (galcon.FPS*50.0)

		if done:
			self.timer += 1
			if self.timer > 2: return dialog.Pause(self.game,'good job!',self.parent)
		else:
			self.frame += 1

		if self.frame > self.timeallowed:
			return dialog.Prompt(self.game,'try again?',self.copy,self.parent)

	def copy(self):
		return Level(self.game,self.parent,self.handicap,self.data,self.timeallowed)
	
	def generate(self):
		# set up the player's home planet
		u = self.users[1]
		p = galcon.Planet()
		p.production = 15
		p.ships = 50
		p.user = u
		self.planets.add(p)
		
		# add 25 random planets
		for n in xrange(0,25):
			p = galcon.Planet()
			p.production = rr(0,100)
			p.ships = rr(0,50)
			p.user = self.users[0]
			self.planets.add(p)
		
		# set the scale - this formula works good
		self.options.scale = 1.0/((len(self.planets)/23.0)**0.5)
		
		# running this function syncs the data here with galcon
		# and gets valuable stuff like planet radius figured out, etc
		self.sync()
		
		# get some basic gist of the screen sizing ...
		sz = 16
		top = galcon.BORDER+sz
		left = galcon.PADDING+sz

		# randomize the position of the planets, including the homeworld
		for p in self.planets.values():
			p.pos = rr(left+p.r,galcon.SW-p.r-left),rr(top+p.r,galcon.SH-p.r-top)

	def paint(self,screen):
		tm = (self.timeallowed - self.frame) / FPS
		text = 'time remaining: %d:%02d'%(tm/60, tm%60)
		img = galcon.Font(15).render(text,1,(255,255,255))
		screen.blit(img,((SW-img.get_width())/2,SH-BORDER+(BORDER-img.get_height())/2))

		if self.first:
			self.first = False
			return dialog.Help(self.game,self.help,self.state)

def Level(game,parent,handicap=0,data=None,timeallowed=-1):
	l_ = Level_(game,parent,handicap,data,timeallowed)
	l = galcon.Level(l_)
	l_.state = l

	return l

#####

# for reference purposes: here's the bot version of the level-length tester

class Bot_Timetest(galcon.Bot):
	def __init__(self,level,speed=2):
		self.level = level
		self.speed = speed
		
	def init(self):
		self.frame = FPS/2

		self.maxdist = 0 # longest distance between two planets
		for p in self.level.planets.values():
			for b in self.level.planets.values():
				dist = math.hypot(p.pos[0]-b.pos[0],p.pos[1]-b.pos[1]) / self.level.options.scale
				if dist > self.maxdist: self.maxdist = dist

		self.minfleet = 1 # minimum allowed fleet size
		
	def loop(self):

		# CHECK SPEED

		self.frame += self.speed
		if self.frame < FPS: return
		else: self.frame -= FPS

		# CONDUCT CENSUS
		
		myTargets = [] # target entries have form [reqships, mydist, planet]
		myPlanets = []
		myForces = 0
		myProd = 0

		# here we'll build a few sublists for efficiency
		maxunscaleddist = self.maxdist * self.level.options.scale
		enplanets = []
		frplanets = []
		myfleets = self.level.fleets.values()

		for p in self.level.planets.values():
			if p.user == self:
				myProd += p.production
				if p.ships >= self.minfleet:
					myPlanets.append([0,p])
					frplanets.append(p)
					myForces += p.ships
			elif p.user.team != self.team: enplanets.append(p)
		if len(myPlanets) == 0: return

		for f in self.level.fleets.values():
			if f.user.team == self.team and f.to not in frplanets: myfleets.append(f)

		# okay, census time
		for p in enplanets:
			reqships = 1 + p.ships
			mydist = maxunscaleddist

			for e in frplanets:
				dist = math.hypot(p.pos[0]-e.pos[0],p.pos[1]-e.pos[1])
				if dist < mydist: mydist = dist
			mydist /= self.level.options.scale

			for f in myfleets:
				if f.to != p: continue
				myfleets.remove(f)
				reqships -= f.ships

			if reqships <= 0: continue
			else: myTargets.append([reqships,mydist,p])

		if len(myTargets) == 0: return
		
		# CHOOSE TARGET

		value = -10000
		target = None

		for p in myTargets:
			t = p[1] # how long would it take to get there?
			t += p[0] * 50 * FPS / myProd # measure the cost in terms of how long it takes us to build that cost
			v = p[2].production * 1000.0 / t # find the best production-to-cost ratio

			if target and v < value: continue
			value,target = v,p

		if target == None: return

		# ATTACK TARGET

		for p in myPlanets: p[0] = math.hypot(p[1].pos[0]-target[2].pos[0],p[1].pos[1]-target[2].pos[1]) / (self.level.options.scale)
		myPlanets.sort()

		reqforces = target[0]
		countedprod = 0
		counteddist = 0
		
		forcenum = len(myPlanets)
		i = 0
		while i < forcenum and reqforces > 0:
			p = myPlanets[i]
			i += 1
			if p[1].ships >= reqforces:
				self.level.fleet(self, p[1], target[2], reqforces)
				reqforces = 0
				break
			else:
				self.level.fleet(self, p[1], target[2], p[1].ships)
				reqforces -= p[1].ships
				reqforces -= (p[0] - counteddist) * countedprod / (FPS * 50)
				counteddist = p[0]
				countedprod += p[1].production
