import galcon
import random
rr = random.randrange
import math

import colorscheme
import classic
import dialog
import mapfile

class Level_(classic.Level):
	def __init__(self,game,parent,handicap = 0,data = None):
		self.help = [
			'---------- Wormhole ----------',
			'',
			'The enemy is attacking through a wormhole in',
			'space-time, and we cannot directly fight back.',
			'Secure the cluster and repel every attack.',
			'',
			'Tip:',
			'When defending your territory (whether or not you',
			'have actually captured the planets in it yet),',
			'try to deny your opponent any toehold; wipe him',
			'out as quickly as possible wherever he goes.'
			]
		self.game = game
		self.parent = parent
		self.handicap = handicap
		self.data = data
		self.wormhole = None
		self.wave = 0
		self.numwaves = 13
		self.uvalue = 0
		
	def init(self):
		# set up the player
		u = galcon.Player()
		u.name = 'Player'
		u.color = colorscheme.human
		self.users.add(u)
		self.player = u

		# set up the main enemy
		u = Wormhole_Bot_Lass(self,3.0 / (3 + self.handicap))
		u.name = 'Invader'
		u.color = colorscheme.lass
		self.users.add(u)

		# set up the wormhole
		u = galcon.Bot()
		u.name = 'The Wormhole'
		u.color = colorscheme.wormhole
		self.users.add(u)
		
		if self.data == None:
			self.generate()
			self.data = mapfile.tostring(self)
		else:
			mapfile.fromstring(self,self.data)

		self.wormhole = self.planets.values()[0]
		for p in self.planets.values():
			if p == self.wormhole: continue
			self.uvalue += p.production*20/50.0

		self.sync()
			
		# initialize all the users / bots
		for u in self.users.values():
			u.init()
			
		# misc
		self.timer,self.frame,self.first = 0,0,True
		
	def copy(self):
		return Level(self.game,self.parent,self.handicap,self.data)
	
	def loop(self):
		for u in self.users.values():
			u.loop()
			
		for p in self.planets.values():
			if p.user.n == 0 or p == self.wormhole: continue
			p.ships += p.production / (galcon.FPS*50.0)

		# invaders code
		realplanets = self.planets.values()
		realplanets.remove(self.wormhole)
		if self.frame % (galcon.FPS*20) == 0:
			self.wave += 1
			# self.uvalue should be approximately 750
			# self.wave varies from 1 to 13
			if self.wave <= self.numwaves:
				for n in xrange(0,self.wave):
					f = galcon.Fleet()
					f.user = self.users[2]
					f.from_ = self.wormhole
					f.to = random.choice(realplanets)
					f.ships = self.uvalue/15 + (7 - self.handicap*2)
					if f.to.user == self.users[0]: f.ships += f.to.ships
					self.fleets.add(f)
			
		livingteam,twosides,twofleets = 0,False,False
		for p in self.planets.values():
			if p.user != self.users[3] and not isinstance(p.user,galcon.Neutral):
				if livingteam == 0:
					livingteam = p.user.team
				elif p.user.team != livingteam:
					twosides = True
		if twosides: twofleets = True
		else:
			for f in self.fleets.values():
				if livingteam == 0:
					livingteam = f.user.team
				elif f.user.team != livingteam:
					twofleets = True
		# self.player.team
		if twosides:
			# so long as everyone has planets, the timer stays at thirty seconds
			self.timer = FPS*30
		else:
			# when one side has no planets, the timer starts ticking down
			self.timer -= 1
			if (not twofleets) and (self.timer > FPS*2):
				# if one side is eliminated, lower the timer to two seconds if necessary
				self.timer = FPS*2
			# end the game when the timer reaches zero
			if self.timer < 1:
				if livingteam != self.player.team:
					return dialog.Prompt(self.game,'try again?',self.copy,self.parent)
				elif self.wave > self.numwaves:
					return dialog.Pause(self.game,'good job!',self.parent)

		self.frame += 1

	def fleet(self,user,from_,to,ships):
		if from_.user != user or ships <= 0 or to == self.wormhole: return
		f = galcon.Fleet()
		f.user = user
		f.from_ = from_
		f.to = to
		f.ships = min(ships, from_.ships)
		from_.ships -= f.ships
		self.fleets.add(f)
		
	def redirect(self,user,from_,to):
		if from_.user != user or to == self.wormhole: return
		from_.to = to
		
	def hit(self,fleet,ships,p):
		s = p.ships
		if p == self.wormhole:
			pass
		elif p.user.team == fleet.user.team:
			p.ships += ships
		else:
			p.ships -= ships
			if p.ships < 0:
				p.ships = -p.ships
				p.user = fleet.user
		return p.ships-s
	
	def generate(self):
		# get some basic gist of the screen sizing ...
		sz = 16
		SW,SH = galcon.SW,galcon.SH
		top = galcon.BORDER+sz
		left = galcon.PADDING+sz
		bottom = SH-top
		right = SW-left

		# set up the wormhole
		u = self.users[3]
		p = galcon.Planet()
		p.production = 200
		p.ships = 0
		p.user = u
		self.planets.add(p)
		wormhole = p
		
		# set up player's home planet
		u = self.users[1]
		p = galcon.Planet()
		p.production = 100
		p.ships = 100
		p.user = u
		self.planets.add(p)
		home = p
		
		# add the other planets
		for n in xrange(0,35):
			p = galcon.Planet()
			p.production = rr(0,100)
			p.ships = rr(0,50)
			p.user = self.users[0]
			self.planets.add(p)
		
		# set the scale - this formula works good
		self.options.scale = 1.0/((len(self.planets)/23.0)**0.5)
		
		# running this function syncs the data here with galcon
		# and gets valuable stuff like planet radius figured out, etc
		self.sync()

		# position the planets
		for p in self.planets.values():
			if p == wormhole:
				p.pos = SW/2,SH/2
			else:
				p.pos = rr(left+p.r,galcon.SW-p.r-left),rr(top+p.r,galcon.SH-p.r-top)

		# position the home planet
		a = rr(0,360)
		x = SW/2+(SH/2)*math.cos(a*math.pi*2.0/360.0)
		y = SH/2+(SH/2)*math.sin(a*math.pi*2.0/360.0)
		home.pos = x,y

	def paint(self,screen):
		if self.numwaves > self.wave:
			wl = self.numwaves-self.wave
			tm = 20 - (self.frame % (20 * FPS))/FPS
			text = 'waves remaining: %d     next wave in: %d seconds'%(wl,tm)
			img = galcon.Font(15).render(text,1,(255,255,255))
			screen.blit(img,((SW-img.get_width())/2,SH-BORDER+(BORDER-img.get_height())/2))

		if self.first:
			self.first = False
			return dialog.Help(self.game,self.help,self.state)

class Wormhole_Bot_Lass(galcon.Bot):
	def __init__(self,level,speed=1):
		self.level = level
		self.speed = speed
		
	def init(self):
		self.frame = random.randrange(0,FPS)
		
	def loop(self):

		# CHECK SPEED

		self.frame += self.speed
		if self.frame < FPS: return
		else: self.frame -= FPS

		# CONDUCT CENSUS

		uncountedFleets = self.level.fleets.values()
		
		myTargets = [] # census entries have form [ships, expectedships, object]
		myPlanets = []

		for p in self.level.planets.values():
			if p == self.level.wormhole: continue
			ships = 1
			if p.user.team == self.team: ships -= p.ships
			else: ships += p.ships
			expships = ships
			for f in uncountedFleets:
				if f.to != p: continue
				uncountedFleets.remove(f)
				if f.user.team != self.team:
					ships += f.ships
					expships += f.ships
				else:
					expships -= f.ships
			if ships > 0: myTargets.append([ships,expships, p])
			elif p.user == self:
				if -expships <= p.ships-2: myPlanets.append([-expships, -expships, p])
				else: myPlanets.append([p.ships-2,-expships,p])
		if len(myPlanets) == 0 or len(myTargets) == 0: return

		# CHOOSE SOURCE
		
		orig = None
		for p in myPlanets:
			if orig and p[0] < orig[0]: continue
			orig = p
		if orig == None: return

		# CHOOSE TARGET
		
		to = None
		value = -1000
		for p in myTargets:
			dist = math.hypot(p[2].pos[0]-orig[2].pos[0],p[2].pos[1]-orig[2].pos[1])
			instacost = int(p[1]+p[2].production/(1+dist/(FPS*self.level.options.scale)))
			if instacost > 0 and instacost < orig[0] and p[2].production / p[0] >= 4 * dist / (FPS*self.level.options.scale):
				self.level.fleet(self,orig[2],p[2],instacost)
				orig[0] -= instacost
			v = -p[0] + p[2].production - (dist / 5)
			if to and v < value: continue
			to,value = p,v
		if to == None: return

		# ATTACK
		
		number = orig[0]
		if number > 0: self.level.fleet(self,orig[2],to[2],number)

def Level(game,parent,handicap=0,data=None):
	l_ = Level_(game,parent,handicap,data)
	l = galcon.Level(l_)
	l_.state = l
	return l
	