## Title: thirdbots.py
## Author: ThirdParty
## Description: Currently contains nine bots for Galcon.

#	each bot takes the level as its first argument and a float "speed" as an optional second argument; "speed" is the number of actions per second and defaults to one
#	in rough order of strength:
#		thirdbots.Bot_Newbie - Newbie emulator: sends half of all ships to biggest enemy planet
#		thirdbots.Bot_Scavenger - philhassey's simple Scavenger bot: takes cheap planets, ignoring their productivity
#		thirdbots.Bot_Basic - philhassey's simple Basic bot: a bot which launches attacks in overwhelmingly-large numbers
#		thirdbots.Bot_Kamikaze - philhassey's simple Kamikaze bot: a bot which immediately goes for the kill
#		thirdbots.Bot_Vendetta - just like Kamikaze, except targets the human player first
#		thirdbots.Bot_Exterminator - hybrid of Kamikaze and Copycat
#		thirdbots.Bot_Lass - same attack algorithm as the Basic bot, but able to respond to players' moves
#		thirdbots.Bot_Cobra - a complex and greedy bot
#		thirdbots.Bot_Libra - a complex bot with some situation-sensitivity

import galcon
import math
import random

class Bot_Newbie(galcon.Bot):
	def __init__(self,level,speed=0.5):
		self.level = level
		self.speed = speed
		
	def init(self):
		self.frame = random.randrange(0,FPS)
		
	def loop(self):
		self.frame += self.speed
		if self.frame < FPS: return
		else: self.frame -= FPS

		targ = None
		value = -10000
		for p in self.level.planets.values():
			if p.user.team == self.team or isinstance(p.user, galcon.Neutral): continue
			v = p.production
			if targ and v <= value: continue
			targ,value = p,v
		if targ == None: return
		
		for p in self.level.planets.values():
			if p.user != self or p.ships < 2: continue
			self.level.fleet(self,p,targ,p.ships*50/100)

class Bot_Scavenger(galcon.Bot):
	def __init__(self,level,speed=1):
		self.level = level
		self.speed = speed
		
	def init(self):
		self.frame = rrfps(self)
		
	def loop(self):
		self.frame += self.speed
		if self.frame < FPS: return
		else: self.frame -= FPS
		
		from_ = None
		for p in self.level.planets.values():
			if p.user != self: continue
			if from_ and p.ships < from_.ships: continue
			from_ = p
		if from_ == None: return
		
		to = None
		value = -1000
		for p in self.level.planets.values():
			if p.user.team == self.team: continue
			dist = math.hypot(p.pos[0]-from_.pos[0],p.pos[1]-from_.pos[1])
			v = -p.ships - (dist / 5)
			if to and v < value: continue
			to,value = p,v
		if to == None: return
		
		self.level.fleet(self,from_,to,from_.ships*65/100)

class Bot_Basic(galcon.Bot):
	def __init__(self,level,speed=1):
		self.level = level
		self.speed = speed
		
	def init(self):
		self.frame = rrfps(self)
		
	def loop(self):
		self.frame += self.speed
		if self.frame < FPS: return
		else: self.frame -= FPS
		
		from_ = None
		for p in self.level.planets.values():
			if p.user != self: continue
			if from_ and p.ships < from_.ships: continue
			from_ = p
		if from_ == None: return
		
		to = None
		value = -1000
		for p in self.level.planets.values():
			if p.user.team == self.team: continue
			dist = math.hypot(p.pos[0]-from_.pos[0],p.pos[1]-from_.pos[1])
			v = -p.ships + p.production - (dist / 5)
			if to and v < value: continue
			to,value = p,v
		if to == None: return
		
		self.level.fleet(self,from_,to,from_.ships*65/100)

class Bot_Kamikaze(galcon.Bot):
	def __init__(self,level,speed=1):
		self.level = level
		self.speed = speed
		
	def init(self):
		self.frame = rrfps(self)
		
	def loop(self):
		self.frame += self.speed
		if self.frame < FPS: return
		else: self.frame -= FPS
		
		from_ = None
		for p in self.level.planets.values():
			if p.user != self: continue
			if from_ and p.ships < from_.ships: continue
			from_ = p
		if from_ == None: return
		
		to = None
		value = -1000
		for p in self.level.planets.values():
			if p.user.team == self.team: continue
			if p.user.n == 0: continue
			dist = math.hypot(p.pos[0]-from_.pos[0],p.pos[1]-from_.pos[1])
			v = -p.ships + p.production - (dist / 5)
			if to and v < value: continue
			to,value = p,v
		if to == None: return
		
		self.level.fleet(self,from_,to,from_.ships*65/100)

class Bot_Vendetta(galcon.Bot):
	def __init__(self,level,speed=1):
		self.level = level
		self.speed = speed
		
	def init(self):
		self.frame = random.randrange(0,FPS)
		
	def loop(self):
		self.frame += self.speed
		if self.frame < FPS: return
		else: self.frame -= FPS
		
		from_ = None
		for p in self.level.planets.values():
			if p.user != self: continue
			if from_ and p.ships < from_.ships: continue
			from_ = p
		if from_ == None: return
		
		to = None
		value = -1000
		for p in self.level.planets.values():
			if p.user.team != self.level.player.team: continue
			dist = math.hypot(p.pos[0]-from_.pos[0],p.pos[1]-from_.pos[1])
			v = -p.ships + p.production - (dist / 5)
			if to and v < value: continue
			to,value = p,v
		if to == None:
			for p in self.level.planets.values():
				if p.user.team == self.team or isinstance(p.user, galcon.Neutral): continue
				dist = math.hypot(p.pos[0]-from_.pos[0],p.pos[1]-from_.pos[1])
				v = -p.ships + p.production - (dist / 5)
				if to and v < value: continue
				to,value = p,v
		if to == None: return
		
		self.level.fleet(self,from_,to,from_.ships*65/100)

class Bot_Exterminator(galcon.Bot):
	def __init__(self,level,speed=1):
		self.level = level
		self.speed = speed
		
	def init(self):
		self.frame = random.randrange(0,FPS) - FPS
		
	def loop(self):
		# CHECK SPEED

		self.frame += self.speed
		if self.frame < FPS: return
		else: self.frame -= FPS

		# CONDUCT CENSUS

		myTargets = [] # census entries have the form [ships needed, planet]
		myPlanets = []

		uncountedFleets = self.level.fleets.values()

		for p in self.level.planets.values():
			ships = 0
			if p.user.team == self.team: ships -= p.ships
			elif not isinstance(p.user, galcon.Neutral): ships += 1 + p.ships
			for f in uncountedFleets:
				if f.to != p: continue
				uncountedFleets.remove(f)
				if f.user.team == self.team: ships -= f.ships
				else: ships += f.ships
			if ships > 0: myTargets.append([ships, p])
			elif p.user == self and p.ships > 1 and ships < 0: myPlanets.append([ships, p])
		if len(myTargets) == 0 or len(myPlanets) == 0: return

		# MAKE MOVE

		orig = None
		for p in myPlanets:
			if orig:
				if orig[0] < p[0]: continue
			orig = p
		
		targ = None
		value = -1000
		for p in myTargets:
			dist = math.hypot(p[1].pos[0]-orig[1].pos[0],p[1].pos[1]-orig[1].pos[1])
			v = -p[0] + p[1].production - (dist / 5)
			if targ and v < value: continue
			targ,value = p,v
		if targ == None: return

		self.level.fleet(self,orig[1],targ[1],min(orig[1].ships*65/100,-orig[0]))

class Bot_Lass(galcon.Bot):
	def __init__(self,level,speed=1):
		self.level = level
		self.speed = speed
		
	def init(self):
		self.frame = rrfps(self)
		
	def loop(self):

		# CHECK SPEED

		self.frame += self.speed
		if self.frame < FPS: return
		else: self.frame -= FPS

		# CONDUCT CENSUS

		uncountedFleets = self.level.fleets.values()
		
		myTargets = [] # census entries have form [ships, expectedships, object]
		myPlanets = []

		for p in self.level.planets.values():
			ships = 1
			if p.user.team == self.team: ships -= p.ships
			else: ships += p.ships
			expships = ships
			for f in uncountedFleets:
				if f.to != p: continue
				uncountedFleets.remove(f)
				if f.user.team != self.team:
					ships += f.ships
					expships += f.ships
				else:
					expships -= f.ships
			if ships > 0: myTargets.append([ships,expships, p])
			elif p.user == self:
				if -expships <= p.ships-2: myPlanets.append([-expships, -expships, p])
				else: myPlanets.append([p.ships-2,-expships,p])
		if len(myPlanets) == 0 or len(myTargets) == 0: return

		# CHOOSE SOURCE
		
		orig = None
		for p in myPlanets:
			if orig and p[0] < orig[0]: continue
			orig = p
		if orig == None: return

		# CHOOSE TARGET
		
		to = None
		value = -1000
		for p in myTargets:
			dist = math.hypot(p[2].pos[0]-orig[2].pos[0],p[2].pos[1]-orig[2].pos[1])
			instacost = int(p[1]+p[2].production/(1+dist/(FPS*self.level.options.scale)))
			if instacost > 0 and instacost < orig[0] and p[2].production / p[0] >= 4 * dist / (FPS*self.level.options.scale):
				self.level.fleet(self,orig[2],p[2],instacost)
				orig[0] -= instacost
			v = -p[0] + p[2].production - (dist / 5)
			if to and v < value: continue
			to,value = p,v
		if to == None: return

		# ATTACK
		
		number = orig[0]
		if number > 0: self.level.fleet(self,orig[2],to[2],number)

class Bot_Cobra(galcon.Bot):
	def __init__(self,level,speed=1):
		self.level = level
		self.speed = speed
		
	def init(self):
		self.frame = rrfps(self)
		self.maxdist = math.hypot(SH, SW) / self.level.options.scale # the maximum possible distance between two objects
		self.minfleet = 5 # minimum allowed fleet size
		
	def loop(self):

		# CHECK SPEED

		self.frame += self.speed
		if self.frame < FPS: return
		else: self.frame -= FPS

		# CONDUCT CENSUS
		
		myTargets = [] # entries have form [reqships, mydist, endist, object]
		myForces = [] # entries have form [0, availships, position, object, isplanet?]
		totalForces = 0

		# here we'll build a few sublists for efficiency
		maxunscaleddist = self.maxdist * self.level.options.scale
		frplanets = []
		enplanets = []
		for p in self.level.planets.values():
			if p.user.team == self.team: frplanets.append(p)
			elif not isinstance(p.user, galcon.Neutral): enplanets.append(p)
		uncountedFleets = self.level.fleets.values()

		# okay, census time
		for p in self.level.planets.values():
			availships = 0
			reqships = 1
			mydist = maxunscaleddist
			frdist = maxunscaleddist
			endist = maxunscaleddist
			fleetlist = [] # fleetlist entries have form [dist, ships, pos, object]

			if p.user.team == self.team:
				availships += p.ships
				frdist = 0
				if p.user == self: mydist = 0
				else:
					 for e in frplanets:
						if e.user != self: continue
						dist = math.hypot(p.pos[0]-e.pos[0],p.pos[1]-e.pos[1])
						if dist < mydist: mydist = dist
				for e in enplanets:
					dist = math.hypot(p.pos[0]-e.pos[0],p.pos[1]-e.pos[1])
					if dist < endist: endist = dist
			else:
				reqships += p.ships
				for e in frplanets:
					dist = math.hypot(p.pos[0]-e.pos[0],p.pos[1]-e.pos[1])
					if dist < frdist: frdist = dist
					if dist < mydist and e.user == self: mydist = dist
				if not isinstance(p.user, galcon.Neutral): endist = 0
				else:
					for e in enplanets:
						dist = math.hypot(p.pos[0]-e.pos[0],p.pos[1]-e.pos[1])
						if dist < endist: endist = dist

			mydist /= self.level.options.scale

			for f in uncountedFleets:
				if f.to != p: continue
				uncountedFleets.remove(f)
				locat = f.get_rect().center
				dist = math.hypot(p.pos[0]-locat[0],p.pos[1]-locat[1])
				if f.user.team != self.team:
					if dist < endist: endist = dist
					reqships += f.ships
				else:
					if dist < frdist: frdist = dist
					availships += f.ships
					fleetlist.append([dist/self.level.options.scale, f.ships, locat, f])

			frdist /= self.level.options.scale
			endist /= self.level.options.scale

			if endist < frdist: reqships += p.production * (50+frdist-endist) / (FPS * 50)
			elif p.user.team == self.team: reqships -= p.production * (endist-frdist) / (FPS * 50)

			if reqships > availships: # evacuate and designate as a possible target
				for f in fleetlist:
					if f[3].user == self and f[1] >= self.minfleet:
						myForces.append([0, f[1], f[2], f[3], False])
						totalForces += f[1]
				if p.user == self and p.ships >= self.minfleet:
					myForces.append([0, p.ships, p.pos, p, True])
					totalForces += p.ships
				myTargets.append([reqships, mydist, endist, p])
			elif availships > reqships + 2: # designate as a possible source
				availships -= (reqships + 2)
				fleetlist.sort()
				i = len(fleetlist)-1
				while i >= 0:
					if fleetlist[i][1] >= availships: break
					if fleetlist[i][3].user == self:
						myForces.append([0, fleetlist[i][1], fleetlist[i][2], fleetlist[i][3], False])
						totalForces += fleetlist[i][1]
					availships -= fleetlist[i][1]
					i -= 1
				if i < 0 and p.user == self and availships >= self.minfleet and p.ships-1 >= self.minfleet:
					if p.ships-1 <= availships:
						myForces.append([0, p.ships-1, p.pos, p, True])
						totalForces += p.ships-1
					else:
						myForces.append([0, availships, p.pos, p, True])
						totalForces += availships

		if len(myTargets) == 0 or len(myForces) == 0: return

		# CHOOSE TARGET

		value = -10000
		target = None

		for p in myTargets:
			t = p[1]
			v = -p[0]
			if p[1] > 0 and p[2] > 0: # should we take a neutral?
				t += FPS * p[0] * 50 / (p[3].production+1) # it'll take some time to recoop the investment
				if p[1] < p[2]: v += (p[2] - p[1]) * p[3].production / (FPS * 50) # but we should get some free ships out of the deal
			elif p[2] > 0: # should we defend our own planet?
				if t < p[2]: t = p[2] # our ships will have to stay there until the enemy force arrives
			v += (self.maxdist - t) * p[3].production / (FPS * 50) # here's how many ships it will produce in the time it takes to transit the map

			if target and v < value: continue
			value,target = v,p

		if target == None: return

		# ATTACK TARGET

		if target[0] > totalForces: return

		availships = 0
		for f in myForces: f[0] = math.hypot(f[2][0]-target[3].pos[0],f[2][1]-target[3].pos[1]) / (self.level.options.scale)
		myForces.sort()

		reqforces = target[0]
		sentforces = 0
		reqtime = target[2]
		sendlist = []
		i = 0
		forcenum = len(myForces)
		while i < forcenum and reqforces > sentforces:
			sendlist.append(myForces[i])
			sentforces += myForces[i][1]
			if myForces[i][0] > reqtime:
				reqforces += (0.0+myForces[i][0]-reqtime) * target[3].production / (FPS * 50)
				reqtime = myForces[i][0]
			i += 1
		if reqforces > sentforces: return

		for f in sendlist:
			if f[4]:
				if f[3] == target[3]: continue
				if f[1] < self.minfleet * 2:
					self.level.fleet(self, f[3], target[3], f[1])
				else:
					self.level.fleet(self, f[3], target[3], f[1]/2)
					self.level.fleet(self, f[3], target[3], f[1] - f[1]/2)
			else:
				if f[3].to == target[3]: continue
				self.level.redirect(self, f[3], target[3])

class Bot_Libra(galcon.Bot):
	def __init__(self,level,speed=1):
		self.level = level
		self.speed = speed
		
	def init(self):
		self.frame = rrfps(self)

		self.maxdist = 0 # longest distance between two planets
		for p in self.level.planets.values():
			for b in self.level.planets.values():
				dist = math.hypot(p.pos[0]-b.pos[0],p.pos[1]-b.pos[1]) / self.level.options.scale
				if dist > self.maxdist: self.maxdist = dist

		self.minfleet = 5 # minimum allowed fleet size
		
	def loop(self):

		# CHECK SPEED

		self.frame += self.speed
		if self.frame < FPS: return
		else: self.frame -= FPS

		# CONDUCT CENSUS
		
		myTargets = [] # entries have form [reqships, mydist, endist, object]
		quickTargets = [] # entries have same form as myTargets
		myForces = [] # entries have form [0, availships, position, object, isplanet?, iscrashing?]
		totalForces = 0 # size of myForces
		frShip = 0 # number of friendly ships anticipated to survive current conflicts
		enShip = 0 # number of enemy ships anticipated to survive current conflicts
		frProd = 0 # quantity of friendly production expected after current conflicts
		enProd = self.maxdist * 100 / (FPS * 50) # quantity of enemy production expected after current conflicts

		# here we'll build a few sublists for efficiency
		maxunscaleddist = self.maxdist * self.level.options.scale
		frplanets = []
		enplanets = []
		for p in self.level.planets.values():
			if p.user.team == self.team: frplanets.append(p)
			elif not isinstance(p.user, galcon.Neutral): enplanets.append(p)
		uncountedFleets = self.level.fleets.values()

		# okay, census time
		for p in self.level.planets.values():
			availships = 0
			reqships = 0
			frdist = maxunscaleddist
			mydist = maxunscaleddist
			endist = maxunscaleddist
			fleetlist = [] # fleetlist entries have form [dist, ships, pos, object, near_dest?]

			if p.user.team == self.team:
				availships += p.ships
				frdist = 0
				if p.user == self: mydist = 0
				else:
					 for e in frplanets:
						if e.user != self: continue
						dist = math.hypot(p.pos[0]-e.pos[0],p.pos[1]-e.pos[1])
						if dist < mydist: mydist = dist
				frShip += 1 # I'm Feeling Lucky
				for e in enplanets:
					dist = math.hypot(p.pos[0]-e.pos[0],p.pos[1]-e.pos[1])
					if dist < endist: endist = dist
			else:
				reqships += 1 + p.ships
				for e in frplanets:
					dist = math.hypot(p.pos[0]-e.pos[0],p.pos[1]-e.pos[1])
					if dist < frdist: frdist = dist
					if dist < mydist and e.user == self: mydist = dist
				if not isinstance(p.user, galcon.Neutral): endist = 0
				else:
					for e in enplanets:
						dist = math.hypot(p.pos[0]-e.pos[0],p.pos[1]-e.pos[1])
						if dist < endist: endist = dist

			mydist /= self.level.options.scale

			for f in uncountedFleets:
				if f.to != p: continue
				uncountedFleets.remove(f)
				locat = f.get_rect().center
				dist = math.hypot(p.pos[0]-locat[0],p.pos[1]-locat[1])
				if f.user.team != self.team:
					willbeneutral = False
					if dist < endist: endist = dist
					reqships += f.ships
				else:
					if dist < frdist: frdist = dist
					availships += f.ships
					if dist < FPS * 2: fleetlist.append([dist/self.level.options.scale, f.ships, locat, f, True])
					else: fleetlist.append([dist/self.level.options.scale, f.ships, locat, f, False])

			frdist /= self.level.options.scale
			endist /= self.level.options.scale

			if endist < frdist: reqships += p.production * (50+frdist-endist) / (FPS * 50)
			elif p.user.team == self.team: reqships -= p.production * endist / (FPS * 50)

			if reqships > availships: # evacuate and designate as a possible target
				frShip += availships
				enShip += reqships
				if frdist == 0: frProd += endist * p.production / (FPS * 50)
				enProd += (self.maxdist-endist) * p.production / (FPS * 50)
				if endist > 0 and frdist > 0:
					enShip -= p.ships # don't count neutral ships as enemy ships
					enShip -= min(p.ships, reqships - p.ships) # in fact, the enemy will lose the ships he spends on the neutral
					if reqships < p.ships * 2: enProd -= (self.maxdist-endist) * p.production / (FPS * 50) # don't count the planet's production if he's not going to win it
				for f in fleetlist:
					if f[3].user == self and f[1] >= self.minfleet:
						myForces.append([0, f[1], f[2], f[3], False, f[4]])
						totalForces += f[1]
				if p.user == self and p.ships >= self.minfleet:
					myForces.append([0, p.ships, p.pos, p, True, False])
					totalForces += p.ships
				if endist > frdist and reqships <= (endist-frdist) * p.production / (FPS * 50): quickTargets.append([reqships, mydist, endist, p])
				else: myTargets.append([reqships, mydist, endist, p])
			else: # designate as a possible source
				if availships <= reqships + 3: continue
				availships -= (reqships + 3)

				frShip += availships
				if endist == 0: enProd += frdist * p.production / (FPS * 50)
				frProd += (self.maxdist-frdist) * p.production / (FPS * 50)
				fleetlist.sort()
				i = len(fleetlist)-1
				while i >= 0:
					if fleetlist[i][1] >= availships: break
					if fleetlist[i][3].user == self:
						myForces.append([0, fleetlist[i][1], fleetlist[i][2], fleetlist[i][3], False, False])
						totalForces += fleetlist[i][1]
					availships -= fleetlist[i][1]
					i -= 1
				if i < 0 and p.user == self and availships >= self.minfleet and p.ships-1 >= self.minfleet:
					if p.ships-1 <= availships:
						myForces.append([0, p.ships-1, p.pos, p, True, False])
						totalForces += p.ships-1
					else:
						myForces.append([0, availships, p.pos, p, True, False])
						totalForces += availships

		if len(myTargets) + len(quickTargets) == 0 or len(myForces) == 0: return

		# ATTACK QUICK TARGETS
		# these are targets which will essentially pay for themselves, and so don't require any complex decisionmaking

		quickTargets.sort()
		i = len(quickTargets)
		while i > 0:
			i -= 1
			p = quickTargets[i]
			orig = None
			origdist = -1
			for f in myForces:
				if (not f[4]) or f[1] < p[0]: continue
				dist = math.hypot(p[3].pos[0]-f[2][0],p[3].pos[1]-f[2][1])
				if orig and origdist <= dist: continue
				origdist = dist
				orig = f
			if orig:
				self.level.fleet(self, orig[3], p[3], p[0])
				orig[1] -= p[0]
				totalForces -= p[0]
				frShip -= p[0]
				frProd += (self.maxdist - (origdist / self.level.options.scale)) * p[3].production / (FPS * 50)
				if orig[1] < self.minfleet:
					myForces.remove(orig)
					totalForces -= orig[1]
			else:
				myTargets.append(p)

		# CHOOSE MAIN TARGET
		# we want to have good ship balance and good near-future production balance

		value = frShip/(0.0+frShip+enShip) + frProd/(0.0+frProd+enProd) # this is the situation I'll be in if I do nothing
		target = None

		for p in myTargets:
			if p[0] > frShip: continue
			frShipLoss = p[0]
			enShipLoss = 0
			frProdGain = 0
			enProdLoss = 0
			if p[1] <= 0 or p[2] <= 0:
				enShipLoss = p[0]
				if p[1] > 0:
					frProdGain = (self.maxdist - p[1]) * p[3].production / (FPS * 50)
					enProdLoss = (self.maxdist - p[1]) * p[3].production / (FPS * 50)
				else:
					frProdGain = (self.maxdist - p[2]) * p[3].production / (FPS * 50)
					enProdLoss = (self.maxdist - p[2]) * p[3].production / (FPS * 50)
			else: # if the planet is currently neutral:
				frProdGain = (self.maxdist - p[1]) * p[3].production / (FPS * 50)
				if p[0] > p[3].ships * 2:
					enShipLoss = p[0] - (2 * p[3].ships)
					enProdLoss = (self.maxdist - max(p[1],p[2])) * p[3].production / (FPS * 50)
				elif p[0] > p[3].ships:
					enShipLoss = p[3].ships - p[0]

			v = (frShip-frShipLoss)/(0.0+(frShip-frShipLoss)+(enShip-enShipLoss)) + (frProd+frProdGain)/(0.0+(frProd+frProdGain)+(enProd-enProdLoss))
			if v < value: continue
			value,target = v,p

		if target == None: return

		# ATTACK MAIN TARGET
		# we'll attack it with whatever forces are nearest it

		if target[0] > totalForces: return

		availships = 0
		for f in myForces: f[0] = math.hypot(f[2][0]-target[3].pos[0],f[2][1]-target[3].pos[1]) / (self.level.options.scale)
		myForces.sort()

		reqforces = target[0]
		sentforces = 0
		reqtime = target[2]
		sendlist = []
		i = 0
		numForces = len(myForces)
		forePlanet = None
		rearForce = None
		rearSize = self.minfleet + 2
		while i < numForces:
			if forePlanet == None and myForces[i][4]: forePlanet = myForces[i][3]
			if reqforces > sentforces: # we'll send as many ships as we need from as close as we can
				sendlist.append(myForces[i])
				sentforces += myForces[i][1]
				if myForces[i][0] > reqtime:
					reqforces += (0.0+myForces[i][0]-reqtime) * target[3].production / (FPS * 50)
					reqtime = myForces[i][0]
			elif myForces[i][4] and myForces[i][1] >= rearSize: # find the spot in the rear where lots of ships are sitting
				rearForce = myForces[i]
				rearSize = myForces[i][1]
			elif (not myForces[i][4]) and myForces[i][5]: # in some situations, ships about to crash unsuccessfully into a planet should be diverted
				if frShip < enShip or isinstance(myForces[i][3].to.user, galcon.Neutral):
					self.level.redirect(self, myForces[i][3], target[3])
			i += 1
		if reqforces <= sentforces:
			for f in sendlist:
				if f[4]:
					if f[3] == target[3]: continue
					n = f[1]
					while n >= self.minfleet * 2:
						self.level.fleet(self, f[3], target[3], n/2)
						n -= n/2
					self.level.fleet(self, f[3], target[3], n)
				else:
					if f[3].to == target[3]: continue
					self.level.redirect(self, f[3], target[3])

		# MOVE SOME SHIPS TOWARD THE FRONT
		# always attacking with the nearest can leave ships stuck at the rear, so don't let them sit there

		if forePlanet and rearForce and forePlanet != rearForce[3]:
			n = rearForce[1] - 2
			while n >= self.minfleet * 2:
				self.level.fleet(self, rearForce[3], forePlanet, n/2)
				n -= n/2
			self.level.fleet(self, rearForce[3], forePlanet, n)

##### ##### #####

def rrfps(me):
	# RANDOM: strategically best
	return random.randrange(0,FPS)

	# DETERMINISTIC/CHAOTIC: good for testing bots while simulating randomness
#	x = 0
#	for p in me.level.planets.values():
#		if p.user == me:
#			x = int(p.pos[0] + 3 * p.pos[1]) % FPS; break
#	return x

	# DETERMINISTIC/ORDERLY: sequences the bots evenly along the second
#	userlist = []
#	for u in me.level.users.values():
#		if isinstance(u, galcon.Neutral) or isinstance(u, galcon.Player): continue
#		userlist.append(u)
#	i = len(userlist)
#	while i > 0:
#		i -= 1
#		if userlist[i] == me: break
#	x = i * FPS / len(userlist)
#	return FPS - (x + 1)