## Author: ThirdParty
## License: Public Domain
## Description: Bot for the "Trade Places" tutorial.

import galcon
import math
import random

class Bot(galcon.Bot):
	def __init__(self,level,speed=2):
		self.level = level
		self.speed = speed
		
	def init(self):
		self.frame = random.randrange(0,FPS)
		self.firstmove = True
		self.bluecol = False
		self.redcol = False

	def loop(self):
		self.frame += self.speed
		if self.frame < FPS: return
		else: self.frame -= FPS
		
		NEUTRAL = self.level.users[0]
		if self.level.users[1] == self: ENEMY = self.level.users[2]
		else: ENEMY = self.level.users[1]
		BLUEHOME = self.level.planets.values()[0]
		REDHOME = self.level.planets.values()[1]
		BLUECOL = self.level.planets.values()[2]
		REDCOL = self.level.planets.values()[3]

		# if this is the first move, send 40 ships to attack enemy (this is a serious error, for blue)
		if self.firstmove:
			self.firstmove = False
			if REDHOME.user == self:
				self.level.fleet(self, REDHOME, BLUEHOME, 40)
			elif BLUEHOME.user == self:
				self.level.fleet(self, BLUEHOME, REDHOME, 40)

		# junk planets should fortify the blue homeworld if they've somehow gotten overfull
		for p in (self.level.planets.values()[4], self.level.planets.values()[5], self.level.planets.values()[6]):
			if p.user != self or p.ships < 7: continue
			self.level.fleet(self, p, BLUEHOME, min(p.ships - 5, p.ships/2))

		# consider taking colonies if I own the relevant worlds

		if not self.bluecol:
			if BLUECOL.user != NEUTRAL: self.bluecol = True
		if not self.redcol:
			if REDCOL.user != NEUTRAL: self.redcol = True

		if REDHOME.user == self and REDHOME.ships >= REDCOL.ships+1 and not self.redcol:
			enemynear = 0
			for f in self.level.fleets.values():
				if f.user != ENEMY: continue
				pos = f.get_rect()
				dist = math.hypot(pos.center[0]-REDCOL.pos[0],pos.center[1]-REDCOL.pos[1])
				if dist >= 200: continue
				enemynear += f.ships
			if enemynear < REDHOME.ships - REDCOL.ships+1:
				self.redcol = True
				self.level.fleet(self, REDHOME, REDCOL, REDCOL.ships+1)

		if BLUEHOME.user == self and BLUEHOME.ships >= BLUECOL.ships+1 and not self.bluecol:
			enemynear = 0
			for f in self.level.fleets.values():
				if f.user != ENEMY: continue
				pos = f.get_rect()
				dist = math.hypot(pos.center[0]-BLUECOL.pos[0],pos.center[1]-BLUECOL.pos[1])
				if dist >= 200: continue
				enemynear += f.ships
			if enemynear < BLUEHOME.ships - BLUECOL.ships+1:
				self.bluecol = True
				self.level.fleet(self, BLUEHOME, BLUECOL, BLUECOL.ships+1)

		# if the enemy owns both blue planets, attack all-out
		if BLUEHOME.user == ENEMY and BLUECOL.user == ENEMY:
			for p in self.level.planets.values():
				if p.user != self or p.ships < 5: continue
				if random.randint(1,10) <= 6: t = BLUEHOME
				else: t = BLUECOL
				self.level.fleet(self, p, t, max(5,p.ships/2))
			return

		# count ships and send extras to attack enemy
		myships = 0
		enships = 0

		for p in self.level.planets.values():
			if p.user == self: myships += p.ships
			elif p.user == ENEMY: enships += p.ships
		for f in self.level.fleets.values():
			if f.user == ENEMY: enships += f.ships

		if myships - (enships + 5) > 2:
			t = None
			f = None
			for p in self.level.planets.values():
				if p.user == self and p.ships > 10 and (f == None or f.ships < p.ships): f = p
				elif p.user == ENEMY and t == None: t = p
			if f != None and t != None:
				self.level.fleet(self, f, t, 5)

		# traffic between homeworlds and their colonies

		if BLUECOL.user == self and BLUEHOME.user == ENEMY and BLUECOL.ships >= 4:
			self.level.fleet(self, BLUECOL, BLUEHOME, BLUECOL.ships / 2)
		if REDCOL.user == self and REDHOME.user == ENEMY and REDCOL.ships >= 4:
			self.level.fleet(self, REDCOL, REDHOME, REDCOL.ships / 2)
		if BLUEHOME.user == self and BLUECOL.user == ENEMY and BLUEHOME.ships >= 4:
			self.level.fleet(self, BLUEHOME, BLUECOL, BLUEHOME.ships / 2)
		if REDHOME.user == self and REDCOL.user == ENEMY and REDHOME.ships >= 4:
			self.level.fleet(self, REDHOME, REDCOL, REDHOME.ships / 2)

		if BLUEHOME.user == self and BLUECOL.user == self:
			q = BLUEHOME.ships - BLUECOL.ships
			for f in self.level.fleets.values():
				if f.user != ENEMY: continue
				if f.to == BLUEHOME: q -= f.ships
				elif f.to == BLUECOL: q += f.ships
			if q >= 4 and BLUEHOME.ships >= 4:
				self.level.fleet(self, BLUEHOME, BLUECOL, min(q/2, BLUEHOME.ships-2))
			elif q <= -4 and BLUECOL.ships >= 4:
				self.level.fleet(self, BLUECOL, BLUEHOME, min(-q/2, BLUEHOME.ships-2))

		if REDHOME.user == self and REDCOL.user == self:
			q = REDHOME.ships - REDCOL.ships
			for f in self.level.fleets.values():
				if f.user != ENEMY: continue
				if f.to == REDHOME: q -= f.ships
				elif f.to == REDCOL: q += f.ships
			if q >= 4 and REDHOME.ships >= 4:
				self.level.fleet(self, REDHOME, REDCOL, min(q/2, REDHOME.ships-2))
			elif q <= -4 and REDCOL.ships >= 4:
				self.level.fleet(self, REDCOL, REDHOME, min(-q/2, REDHOME.ships-2))