## Title: gui.py
## Author: Phil Hassey
## License: LGPL (c) 2007 Phil Hassey
## Description: A simple GUI system that is complete in a 32k file.

import math
import galcon
Rect = galcon.Rect
Font = galcon.Font
class Event:
    def __init__(self,type_):
        self.type = type_
def transform(e,r):
    if r == None: return e
    e2 = e
    if e.type in (MOUSEBUTTONDOWN, MOUSEBUTTONUP):
        e2 = Event(e.type)
        e2.pos = e.pos[0]-r.x,e.pos[1]-r.y
        e2.button = e.button
    if e.type in (MOUSEMOTION,):
        e2 = Event(e.type)
        e2.pos = e.pos[0]-r.x,e.pos[1]-r.y
        e2.buttons = e.buttons
    return e2

FOCUS,BLUR,ENTER,EXIT,CLICK,CHANGE = 'focus','blur','enter','exit','click','change'

def theme_x():
    img = galcon.Surface((14,14))
    img.fill((0,0,0,0))
    c = (0xFF,0xFF,0xFF)
    a,b,w = 2,11,3
    img.line(c,(a,a),(b,b),w)
    img.line(c,(a,b),(b,a),w)
    img.line(c,(a,a),(b,b),w)
    img.line(c,(a,b),(b,a),w)
    return img

c_focus = (0xCC,0xFF,0xFF)
theme = {
    ##############
    'label':{
        'font':Font(16),
        'color':(0xFF,0xFF,0xFF),
        },
        
    ##############
    'document':{
        'font':Font(16),
        'color':(0xFF,0xFF,0xFF),
        },
        
    ##############
    'button':{
        'border':2,
        'padding':2,
        'margin':2,
        },
    'button:':{
        'background':(0x00,0x88,0x88),
        'border-color':(0x00,0xFF,0xFF),
        },
    'button:focus':{
        'background':(0x00,0xAA,0xAA),
        'border-color':c_focus,
        },
    'button:hover':{
        'background':(0x00,0xAA,0xAA),
        },
    'button:down':{
        'background':c_focus, #(0x00,0xFF,0xFF),
        },
    'button label':{
        'font':Font(16),
        'color':(0x00,0xFF,0xFF),
        },
    
    ##############
    'input':{
        'border':2,
        'padding':2,
        'margin':2,
        'background':(0x00,0x88,0x88),
        'width':120,
        'font':Font(16),
        'color':(0xFF,0xFF,0xFF),
        },
    'input:':{
        'background':(0x00,0x88,0x88),
        'border-color':(0x00,0xFF,0xFF),
        },
    'input:focus':{
        'background':(0x00,0xAA,0xAA),
        'border-color':c_focus,
        },
        
    ##############
    'checkbox':{
        'border':2,
        'margin':2,
        'padding':1,
        'width':14,
        'height':14,
        'icon':theme_x(),
        },
    'checkbox:':{
        'background':(0x00,0x88,0x88),
        'border-color':(0x00,0xFF,0xFF),
        },
    'checkbox:hover':{
        'background':(0x00,0xAA,0xAA),
        },
    'checkbox:focus':{
        'border-color':c_focus,
        },
        
    ##############
    'radio':{
        'border':2,
        'margin':2,
        'padding':1,
        'width':14,
        'height':14,
        'icon':theme_x(),
        },
    'radio:':{
        'background':(0x00,0x88,0x88),
        'border-color':(0x00,0xFF,0xFF),
        },
    'radio:hover':{
        'background':(0x00,0xAA,0xAA),
        },
    'radio:focus':{
        'border-color':c_focus,
        },
        
    ##############
    'slider':{
        'border':2,
        'margin':2,
        'color':(0xFF,0xFF,0xFF),
        'font':Font(14),
        'width':120,
        'height':16,
        'size':32,
        },
    'slider:':{
        'background':(0x00,0x88,0x88),
        'border-color':(0x00,0xFF,0xFF),
        'bar':(0x00,0xFF,0xFF),
        },
    'slider:hover':{
        'background':(0x00,0xAA,0xAA),
        },
    'slider:focus':{
        'background':(0x00,0xAA,0xAA),
        'border-color':c_focus,
        'bar':c_focus,
        },
        
    }
    
def theme_resize(w,m):
    def func(width=None,height=None):
        s = w.style.get
        top,right,bottom,left = 0,0,0,0
        left += s('padding-left',s('padding',0))
        left += s('border-left',s('border',0))
        left += s('margin-left',s('margin',0))
        top += s('padding-top',s('padding',0))
        top += s('border-top',s('border',0))
        top += s('margin-top',s('margin',0))
        right += s('padding-right',s('padding',0))
        right += s('border-right',s('border',0))
        right += s('margin-right',s('margin',0))
        bottom += s('padding-bottom',s('padding',0))
        bottom += s('border-bottom',s('border',0))
        bottom += s('margin-bottom',s('margin',0))
        
        # get the widget to give its w & h
        if width != None and height != None:
            ww,hh = m(width-(left+right),height-(top+bottom))
        else:
            ww,hh = m(width,height)
        if width == None: width = left+ww+right
        if height == None: height = top+hh+bottom
        
        align = s('align',0)
        diff = width - (left + ww + right)
        inc = diff*(align+1)/2
        xx = left + inc
        
        align = s('valign',0)
        diff = height - (top + hh + bottom)
        inc = diff*(align+1)/2
        yy = top + inc
        
        w.t_rect = Rect(xx,yy,ww,hh)
        return width,height
    
    return func

def theme_paint(w,m):
    def func(screen):
        # load up our style goodies
        s = w.style.get
        
        # calculate margin box
        r = w.c_rect
        x1 = 0 + s('margin-left',s('margin',0))
        y1 = 0 + s('margin-top',s('margin',0))
        x2 = r.w - s('margin-right',s('margin',0))
        y2 = r.h - s('margin-bottom',s('margin',0))
        
        # draw background
        c = s('background',None)
        if isinstance(c,tuple):
            screen.fill(c,(x1,y1,x2-x1,y2-y1))
        elif c != None:
            screen.blit(c.subsurface(0,0,x2-x1,y2-y1),(x1,y1))
            
        # draw border box
        c = s('border-color',None)
        if c != None:
            size = s('border-left',s('border',0))
            if size: screen.fill(c,Rect(x1,y1,size,y2-y1))
            size = s('border-right',s('border',0))
            if size: screen.fill(c,Rect(x2-size,y1,size,y2-y1))
            size = s('border-top',s('border',0))
            if size: screen.fill(c,Rect(x1,y1,x2-x1,size))
            size = s('border-bottom',s('border',0))
            if size: screen.fill(c,Rect(x1,y2-size,x2-x1,size))
            
        m(screen.subsurface(w.t_rect))
    return func
        

def theme_event(w,m):
    def func(e):
        if e.type is MOUSEBUTTONDOWN:
            w.t_state = True
        elif e.type is MOUSEBUTTONUP:
            w.t_state = False
        elif e.type is ENTER:
            w.t_hover = True
        elif e.type is EXIT:
            w.t_hover = False
        elif e.type is FOCUS:
            w.t_focus = True
        elif e.type is BLUR:
            w.t_focus = False
        e2 = transform(e,w.t_rect)
        r = m(e2)
        return r
    return func

def theme_restyle(w,m):
    def func():
        m()
        w.pseudo = ['']
        if w.t_hover: w.pseudo.append('hover')
        if w.t_focus: w.pseudo.append('focus')
        if w.t_state and w.t_hover and w.t_focus: w.pseudo.append('down')
        w.pseudo = tuple(w.pseudo)
        
        if w.t_element == w.element and w.t_klass == w.klass and w.t_pseudo == w.pseudo: return
        w.t_element,w.t_klass,w.t_pseudo = w.element,w.klass,w.pseudo
        
        key = '%s'%(w.element)
        if key in theme: add_style(w.style,theme[key])
        for pseudo in w.pseudo:
            key = '%s:%s'%(w.element,pseudo)
            if key in theme: set_style(w.style,theme[key])
        if w.klass != None:
            key = '%s.%s'%(w.element,w.klass)
            if key in theme: set_style(w.style,theme[key])
            for pseudo in w.pseudo:
                key = '%s.%s:%s'%(w.element,w.klass,pseudo)
                if key in theme: set_style(w.style,theme[key])
        set_style(w.t_style,w.style)
    return func
    

def theme_deco(w):
    w.t_rect = None
    w.t_state = False
    w.t_hover = False
    w.t_focus = False
    w.restyle = theme_restyle(w,w.restyle)
    w.resize = theme_resize(w,w.resize)
    w.paint = theme_paint(w,w.paint)
    w.event = theme_event(w,w.event)
    w.t_element = None
    w.t_klass = None
    w.t_pseudo = None
    w.t_style = {}
    w.restyle()
    
    for k in ['width','height']:
        w.style.setdefault(k,0)

def get_style(theme_style,**params):
    style = theme_style.copy()
    for k in ['width','height','background','align','valign']:
        if k in params: style[k] = params[k] 
    add_style(style,params.get('style',{}))
    return style
def add_style(style1,style2):
    for k,v in style2.items():
        if k not in style1: style1[k] = v
def set_style(style1,style2):
    for k,v in style2.items():
        style1[k] = v

def widget_event(w,m):
    def func(e):
        # top level container set up ..
        if w.container == None:
            Globals['widget'] = w
        
        # carry on ...
        r = m(e)
        if e.type is CLICK: w.onclick()
        elif e.type is CHANGE: w.onchange()
        
        if r != True and w.focusable:
            if e.type is KEYDOWN and e.key in (K_TAB,K_UP,K_DOWN,K_LEFT,K_RIGHT):
                focus = None
                cur = Globals['widget']
                widgets,todo = [],[cur]
                while len(todo):
                    cur = todo.pop(0)
                    if cur.container != None:
                        p = cur.container
                        x = p.g_rect.x+cur.c_rect.x+cur.t_rect.x
                        y = p.g_rect.y+cur.c_rect.y+cur.t_rect.y
                    else:
                        x,y = 0,0
                    cur.g_rect = Rect(x,y,cur.t_rect.w,cur.t_rect.h)
                    if isinstance(cur,Container):
                        todo = cur.widgets + todo
                    elif cur.focusable:
                        widgets.append(cur)
                        
                widgets_ = widgets[:]
                if e.key == K_TAB:
                    i = widgets.index(w)
                    if (e.mod & KMOD_SHIFT) != 0:
                        widgets,n = widgets[:i],-1
                    else: widgets,n = widgets[i+1:],0
                    if len(widgets):
                        focus = widgets.pop(n)
                else:
                    widgets.remove(w)
                    cr = w.g_rect
                    cx,cy = cr.centerx,cr.centery
                    dist = 0xffffffff
                    
                    if e.key == K_UP: dx_,dy_ = 0,-1
                    elif e.key == K_RIGHT: dx_,dy_ = 1,0
                    elif e.key == K_DOWN: dx_,dy_ = 0,1
                    elif e.key == K_LEFT: dx_,dy_ = -1,0
                    
                    for next in widgets:
                        nr = next.g_rect
                        nx,ny = nr.centerx,nr.centery
                        if dx_ > 0:
                            if nr.left < cr.right: continue
                            nx = nr.left
                        if dx_ < 0:
                            if nr.right > cr.left: continue
                            nx = nr.right
                        if dy_ > 0:
                            if nr.top < cr.bottom: continue
                            ny = nr.top
                        if dy_ < 0:
                            if nr.bottom > cr.top: continue
                            ny = nr.bottom
                        dx,dy = nx-cx,ny-cy
                        nd = dx*dx+dy*dy
                        if nd < dist: focus,dist = next,nd
                
                if focus != None:
                    focus.focus()
                return True
        return r
    return func
        
def widget_change(w,m):
    def func():
        m()
        w.event(Event(CHANGE))
    return func
    
def widget_paint(w,m):
    def func(screen):
        # for top level item, auto calc rect
        if w.c_rect == None:
            w.c_rect = Rect(0,0,screen.get_width(),screen.get_height())
            
        # for top level item, be sure to restyle and resize before painting
        if w.container == None:
            w.restyle()
            w.resize() # or - just put it in init()
        
        r = m(screen)
        return r
    return func

def widget_deco(w):
    w.event = widget_event(w,w.event)
    w.change = widget_change(w,w.change)
    w.paint = widget_paint(w,w.paint)
    w.g_rect = None
    
class Widget:
    """Base for all widgets.
    
    Widget(**params)
    
    name - name of widget - used by forms
    value - initial value of widget - used by forms
    focusable - True if the widget has any use for focus
    style - a dict of style parameters - used by theme
    element - name of element ('input', 'slider', etc) - used by theme
    klass - class of the element ('on', 'off', etc) - used by theme
    """
    def __init__(self,**params):
        self.container = None
        self.c_rect = None
        self.form = None
        if 'name' in params:
            self.name = params['name']
            self.form = Globals['form']
            self.form[self.name] = self
        self.value = None
        if 'value' in params:
            self.value = params['value']
        self.focusable = params.get('focusable',False)
        
        self.style = get_style({},**params)
        self.element = params.get('element',None)
        self.klass = params.get('klass',None)
        self.pseudo = None
        theme_deco(self)
        widget_deco(self)
    
    def restyle(self): # handle style updates ..
        pass
    def resize(self,width=None,height=None):
        """Resize the widget.
        
        resize(width=None,height=None)
        
        The passed in width and height are optional larger sizes that
        the widget may use.  The widget must return a width and a height
        that it would prefer.
        """
        return self.style['width'],self.style['height']
    def paint(self,screen):
        """Paint the widget.
        
        paint(screen)
        """
        pass
    def event(self,e):
        """Handle an event.
        
        event(e)
        
        After this is called, onclick may be auto-called.
        """
        pass
    
    def change(self): # call this if you change some .value or whatever
        """Handle a change to the value.
        
        change()
        
        Anytime the value is changed call change().  In the change
        method handle any image / data updates that need to happen
        due to the change.  onchange() will be auto-called.
        """
    
    def onclick(self): pass
    def onchange(self): pass
    
    def focus(self): # if you call this on a container, it won't work . yeah.
        """Give this widget the focus.
        
        focus() - this method is final.
        """
        # blur
        Globals['widget'].event(Event(BLUR))
        cur = self
        while cur != None:
            cur.event(Event(FOCUS))
            prev = cur
            cur = cur.container
            if cur != None:
                cur.myfocus = prev
    
Globals = {}
class Form(dict):
    """A Form object - a dictionary.
    
    Form()
    """
    def __init__(self):
        Globals['form'] = self

class Group(Widget):
    """A (non-visible) Group widget - used for grouping Radio buttons.
    
    Group(name)
    
    Don't add this widget to your Container.  Just create it
    and use it for building Radio button sets.
    """
    def __init__(self,name,**params):
        self.widgets = []
        params['name'] = name
        Widget.__init__(self,**params)
    def change(self):
        for w in self.widgets:
            w.change()

class Label(Widget):
    """A textual label.
    
    Label(text)
    """
    def __init__(self,text,**params):
        self.text = text
        params.setdefault('element','label')
        Widget.__init__(self,**params)
        self.change()
    def change(self):
        self.surf = self.style['font'].render(self.text,1,self.style['color'])
    def resize(self,width=None,height=None):
        return self.surf.get_width(),self.surf.get_height()
    def paint(self,screen):
        screen.blit(self.surf,(0,0))
        
    
class Spacer(Widget):
    """A invisible spacer.
    
    Spacer(width,height)
    """
    def __init__(self,width,height,**params):
        params['width'] = width
        params['height'] = height
        params.setdefault('element','spacer')
        Widget.__init__(self,**params)
    
class Image(Widget):
    """An image.
    
    Image(surf)
    """
    def __init__(self,surf,**params):
        params.setdefault('element','image')
        Widget.__init__(self,**params)
        params.setdefault('width',surf.get_width())
        params.setdefault('height',surf.get_height())
        self.surf = surf.scale((params['width'],params['height']))
    def paint(self,screen):
        screen.blit(self.surf,(0,0))
    def resize(self,width=None,height=None):
        return self.surf.get_width(),self.surf.get_height()
    
class Container(Widget):
    """The container widget.
    
    Container()
    """
    def __init__(self,**params):
        self.widgets = []
        self.myfocus = None
        self.myhover = None
        params.setdefault('element','container')
        Widget.__init__(self,**params)
    def paint(self,screen):
        for w in self.widgets:
            w.paint(screen.subsurface(w.c_rect))
    def add(self,w,pos):
        """Add a widget to this container.
        
        add(widget,pos)
        
        pos - x,y position of the widget
        """
        w.container = self
        self.widgets.append(w)
        w.c_rect = Rect(pos[0],pos[1],0,0)
    def event(self,e):
        # handle focus / blur events
        prev,cur = self.myfocus,self.myfocus
        if e.type is MOUSEBUTTONDOWN:
            cur = None
            for w in self.widgets:
                if w.c_rect.collidepoint(e.pos): cur = w
        if e.type is BLUR: cur = None
        if prev != None and cur != prev: prev.event(Event(BLUR))
        if cur != None and cur != prev: cur.event(Event(FOCUS))
        self.myfocus = cur
            
        # handle enter / exit events
        prev,cur = self.myhover,self.myhover
        if e.type is MOUSEMOTION:
            cur = None
            for w in self.widgets:
                if w.c_rect.collidepoint(e.pos): cur = w
        if e.type is EXIT: cur = None
        if prev != None and cur != prev: prev.event(Event(EXIT))
        if cur != None and cur != prev: cur.event(Event(ENTER))
        self.myhover = cur
        
        # if we're hovering, do that ...
        if self.myhover != None and e.type is MOUSEMOTION:
            w = self.myhover
            e2 = transform(e,w.c_rect)
            w.event(e2)
            
        # otherwise .. do whatever :)
        if self.myfocus != None:
            w = self.myfocus
            e2 = transform(e,w.c_rect)
            w.event(e2)
            
            if self.container == None:
                if e.type is MOUSEBUTTONUP and self.myfocus == self.myhover:
                    w.event(Event(CLICK))
                if e.type is KEYUP and e.key == K_SPACE:
                    w.event(Event(CLICK))
    
    def resize(self,width=None,height=None):
        ww,hh = 0,0
        for w in self.widgets:
            w.c_rect.w,w.c_rect.h = w.resize()
            ww = max(ww,w.c_rect.x+w.c_rect.w)
            hh = max(hh,w.c_rect.y+w.c_rect.h)
        return ww,hh
        
    def restyle(self):
        for w in self.widgets: w.restyle()
    
def div(a,b,c):
    v,r = a/b, a%b
    if r != 0 and (c%b)<r: v += 1
    return v
    
class Table(Container):
    """Table layout manager.
    
    Table()
    """
    def __init__(self,**params):
        params.setdefault('element','table')
        Container.__init__(self,**params)
        self.row = -1
        self.col = 0
        self.grid = {}
        
    def tr(self):
        """Create a new table row.
        
        tr()
        """
        self.row,self.col = self.row+1,0
        
    def td(self,w,colspan=1,rowspan=1):
        """Create a new table cell.
        
        td(widget,colspan=1,rowspan=1)
        
        Your widget will be auto-resized to fill the cell.  If
        that is not desirable, wrap your widget via Box(widget).
        """
        grid = self.grid
        self.row = max(0,self.row) # force the initial tr()
        col,row = self.col,self.row
        while grid.get((col,row),None) != None: col += 1
        w.b_cell = (col,row)
        w.b_colspan = colspan
        w.b_rowspan = rowspan
        for y in xrange(w.b_cell[1],w.b_cell[1]+w.b_rowspan):
            for x in xrange(w.b_cell[0],w.b_cell[0]+w.b_colspan):
                grid[(x,y)] = w
        self.col = col + colspan
        self.add(w,(0,0))
        
    def resize(self,width=None,height=None):
        # figure out how many rows, cols we have & resize widgets
        cols,rows = 0,0
        for w in self.widgets:
            cols = max(cols,w.b_cell[0]+w.b_colspan)
            rows = max(rows,w.b_cell[1]+w.b_rowspan)
            w.c_rect.w,w.c_rect.h = w.resize()
        widgets = self.widgets[:]
        
        # this widget represents the table widget :)
        w = Widget()
        w.b_cell = (0,0)
        w.b_colspan,w.b_rowspan = cols,rows
        w.c_rect = Rect(0,0,self.style.get('width',0),self.style.get('height',0))
        widgets.append(w)
        
        # set up the initial cell sizes
        cols = [0 for n in xrange(0,cols)]
        rows = [0 for n in xrange(0,rows)]
        
        # size out our cols
        def sort_by_colspan(a,b):
            return a.b_colspan-b.b_colspan
        widgets.sort(sort_by_colspan)
        for w in widgets:
            v = sum([cols[x] for x in xrange(w.b_cell[0],w.b_cell[0]+w.b_colspan)])
            if v >= w.c_rect.w: continue
            diff = w.c_rect.w - v
            for x in xrange(w.b_cell[0],w.b_cell[0]+w.b_colspan):
                cols[x] += div(diff,w.b_colspan,x)
            
        # size out our rows
        def sort_by_rowspan(a,b):
            return a.b_rowspan-b.b_rowspan
        widgets.sort(sort_by_rowspan)
        for w in widgets:
            v = sum([rows[x] for x in xrange(w.b_cell[1],w.b_cell[1]+w.b_rowspan)])
            if v >= w.c_rect.h: continue
            diff = w.c_rect.h - v
            for x in xrange(w.b_cell[1],w.b_cell[1]+w.b_rowspan): 
                rows[x] += div(diff,w.b_rowspan,x)
            
        # calc positions
        ww = 0
        cols_pos = []
        for n in xrange(0,len(cols)):
            cols_pos.append(ww)
            ww += cols[n]
        
        hh = 0
        rows_pos = []
        for n in xrange(0,len(rows)):
            rows_pos.append(hh)
            hh += rows[n]
            
        # move widgets into position
        for w in self.widgets: #use real widgets this time
            w.c_rect.x,w.c_rect.y = cols_pos[w.b_cell[0]],rows_pos[w.b_cell[1]]
            w2 = sum([cols[x] for x in xrange(w.b_cell[0],w.b_cell[0]+w.b_colspan)])
            h2 = sum([rows[x] for x in xrange(w.b_cell[1],w.b_cell[1]+w.b_rowspan)])
            if (w.c_rect.w,w.c_rect.h) != (w2,h2):
                w.c_rect.w,w.c_rect.h = w.resize(w2,h2)
            
        return ww,hh

        
class Button(Widget):
    """A clickable button.
    
    Button(widget)
    
    widget - a widget to render centered on the button face.
    """
    def __init__(self,widget,**params):
        params.setdefault('element','button')
        params.setdefault('focusable',True)
        Widget.__init__(self,**params)
        if isinstance(widget,str):
            widget = Label(widget)
        self.widget = widget
        
    def resize(self,width=None,height=None):
        w,h = self.widget.resize(width,height)
        return max(self.style['width'],w),max(self.style['height'],h)
    
    def paint(self,screen):
        self.widget.c_rect = self.c_rect
        self.widget.paint(screen)
        
class Input(Widget):
    """A text input.
    
    Input(name)
    """
    def __init__(self,name,**params):
        params['name'] = name
        params.setdefault('value','')
        params.setdefault('element','input')
        params.setdefault('focusable',True)
        Widget.__init__(self,**params)
        self.change()
        
    def event(self,e):
        if e.type is KEYDOWN and e.key == K_BACKSPACE:
            self.value = self.value[:-1]
            self.change()
        elif e.type is KEYDOWN:
            c = galcon.key_get_char(e.unicode)
            self.value += c
            self.change()

    def change(self):
        self.surf = self.style['font'].render(self.value+'|',1,self.style['color'])

    def paint(self,screen):
        sw =screen.get_width()
        img = self.surf
        x = 0
        ww = img.get_width()
        if ww > sw:
            x = sw-ww
        screen.blit(img,(x,0))
        
    def resize(self,width=None,height=None):
        if width == None: width = self.style['width']
        return width,self.style['font'].get_height()
            
class Checkbox(Widget):
    """A checkbox.
    
    Checkbox(name)
    
    value - True or False.  Defaults to False.
    """
    def __init__(self,name,**params):
        params['name'] = name
        params.setdefault('value',False)
        params.setdefault('element','checkbox')
        params.setdefault('focusable',True)
        Widget.__init__(self,**params)
        self.change()
        
    def event(self,e):
        if e.type is CLICK:
            self.value = not self.value
            self.change()
    
    def change(self):
        self.klass = 'off'
        if self.value: self.klass = 'on'
        
    def paint(self,screen):
        if self.klass == 'on':
            screen.blit(self.style['icon'],(0,0))

class Radio(Widget):
    """Radio button control.
    
    Radio(group,value)
    
    group - the group the radio button is a part of.
    """
    def __init__(self,group,value,**params):
        params['value'] = value
        params.setdefault('element','radio')
        params.setdefault('focusable',True)
        self.group = group
        self.group.widgets.append(self)
        Widget.__init__(self,**params)
        self.change()
        
    def event(self,e):
        if e.type is CLICK:
            self.group.value = self.value
            self.group.change()
            
    def change(self):
        self.klass = 'off'
        if self.value == self.group.value: self.klass = 'on'
        
    def paint(self,screen):
        if self.klass == 'on':
            screen.blit(self.style['icon'],(0,0))

class Slider(Widget):
    """A horizontal numerical slider.
    
    Slider(name,min,max)
    """
    def __init__(self,name,min_,max_,**params):
        params['name'] = name
        params.setdefault('value',None)
        params.setdefault('element','slider')
        params.setdefault('focusable',True)
        Widget.__init__(self,**params)
        self.max_ = max_
        self.min_ = min_
        if self.value == None: self.value = (self.min_+self.max_)/2
        self.change()
        
    def event(self,e):
        if e.type is MOUSEMOTION:
            if not self.t_focus: return
            if 1 not in e.buttons: return
            size = self.style['size']
            x = e.pos[0]
            v = float(x-size/2) * float(self.max_-self.min_) / float(self.myrect.w-size) + self.min_
            self.value = v
            self.change()
        if e.type is KEYDOWN and e.key in (K_RIGHT,):
            self.value += 1
            self.change()
            return True
        if e.type is KEYDOWN and e.key in (K_LEFT,):
            self.value -= 1
            self.change()
            return True
            
    def change(self):
        self.value = int(round(max(self.min_,min(self.max_,self.value))))
        self.surf = self.style['font'].render(str(self.value),1,self.style['color'])
            
    def paint(self,screen):
        size = self.style['size']
        x = (self.value-self.min_) * (self.myrect.w-size) / (self.max_-self.min_) + size/2
        r = Rect(x-size/2,0,size,self.myrect.h)
        screen.fill(self.style['bar'],r)
        img = self.surf
        pos = (x-img.get_width()/2,self.myrect.h/2-img.get_height()/2)
        screen.blit(img,pos)
        
    def resize(self,width=None,height=None):
        if width == None or height == None:
            width,height = Widget.resize(self,width,height)
        self.myrect = Rect(0,0,width,height)
        return width,height
    
class Document(Widget):
    """A auto-wrapped text document.
    
    Document(content,width)
    
    content - plain text, all whitespace is respected; \t = 4 spaces.
    """
    def __init__(self,content,width,**params):
        self.content = content
        params.setdefault('width',width)
        params.setdefault('element','document')
        Widget.__init__(self,**params)
        self.change()
    def change(self):
        fnt = self.style['font']
        c = self.style['color']
        w = self.style['width']
        content = self.content.replace('\t','    ')
        sw,sh = fnt.size(' ')
        lines = self.lines = []
        for line in content.split('\n'):
            first,prev = True,''
            for word in line.split(' '):
                text = prev + word
                iw,ih = fnt.size(text)
                if not first and iw >= w:
                    lines.append(fnt.render(prev,1,c))
                    first,prev = False,word+' '
                else:
                    first,prev = False,text+' '
            lines.append(fnt.render(prev,1,c))
    def resize(self,width=None,height=None):
        return self.style['width'],len(self.lines)*self.lines[0].get_height()
    def paint(self,screen):
        align = self.style.get('align',0)
        y = 0
        for img in self.lines:
            dx = screen.get_width()-img.get_width()
            x = (align+1)*dx/2
            screen.blit(img,(x,y))
            y += img.get_height()
    
class Box(Container):
    """A box container for a single widget.
    
    Box(widget)
    
    This is useful within tables, when auto-resize of the widget
    to fill the cell is not desirable.  Or for just extra boxes.
    """
    def __init__(self,widget,**params):
        params.setdefault('element','box')
        Container.__init__(self,**params)
        self.add(widget,(0,0))
        self.widget = widget
        
    def resize(self,width=None,height=None):
        s = self.style.get
        w = self.widget
        w.c_rect.w,w.c_rect.h = w.resize()
        
        width = max(width,w.c_rect.w,s('width',0))
        height = max(height,w.c_rect.h,s('height',0))
        
        dx = width-w.c_rect.w
        dy = height-w.c_rect.h
        w.c_rect.x = (s('align',0)+1)*dx/2
        w.c_rect.y = (s('valign',0)+1)*dy/2
        
        return width,height

import dialog
class App(galcon.State):
    """A State widget.
    
    App(game,widget,parent)
    
    game - your global game data
    widget - your top level widget
    parent - parent state to return to if they escape / quit
    """
    def __init__(self,game,w,parent):
        self.game = game
        self.widget = w
        self.next = None
        self.parent = parent
    def init(self):
        pass
    def event(self,e):
        if e.type is KEYDOWN and e.key == K_ESCAPE:
            return dialog.Prompt(self.game,'quit?',self.parent,self)
        self.widget.event(e)
    def paint(self,screen):
        #Globals['screen'] = screen
        self.widget.paint(screen)
    def goto(self,state):
        self.next = state
    def loop(self):
        r = None
        if self.next != None:
            r = self.next
            self.next = None
        return r
