## Title: classic.py
## Author: Phil Hassey (modified slightly by ThirdParty)
## License: Public Domain
## Description: The classic Galcon mission.

import galcon
import dialog

class Level(galcon.State):
    # the classic level requires
    # .copy - a copy of this level to retry
    # .timer - set to 0
    # .frame - set to 0
    # .parent - parent to return to 
    # .state - the state we're actually in ..
    
    def loop(self):
        for u in self.users.values():
            u.loop()
            
        for p in self.planets.values():
            if p.user.n == 0: continue
            p.ships += p.production / (galcon.FPS*50.0)
            
        counts = [0,0,0]
        for p in self.planets.values():
            n = 0
            if p.user.team == self.player.team: n = 1
            elif not isinstance(p.user,galcon.Neutral): n = 2 
            counts[n] += 1
        if counts[1] != 0 and counts[2] != 0:
            self.timer = 0
            
        if counts[2] == 0:
            self.timer += 1
            if self.timer == FPS*2:
                return dialog.Pause(self.game,'good job!',self.parent)
        if counts[1] == 0:
            self.timer += 1
            if self.timer == FPS*2:
                return dialog.Prompt(self.game,'try again?',self.copy,self.parent)
            
        self.frame += 1
    
    def fleet(self,user,from_,to,ships):
        if from_.user != user or ships <= 0: return
        ships = min(from_.ships,ships)
        
        f = galcon.Fleet()
        f.user = user
        f.from_ = from_
        f.to = to
        f.ships = ships
        from_.ships -= ships
        self.fleets.add(f)
        
    def redirect(self,user,from_,to):
        if from_.user != user: return
        from_.to = to
        
    def hit(self,fleet,ships,p):
        s = p.ships
        if p.user.team == fleet.user.team:
            p.ships += ships
        else:
            p.ships -= ships
            if p.ships < 0:
                p.ships = -p.ships
                p.user = fleet.user
        return p.ships-s
    
    def event(self,e):
        if e.type is KEYDOWN and e.key == K_ESCAPE:
            return dialog.Choice(self.game,None,[
                ('continue',K_c,self.state),
                ('restart',K_r,self.copy),
                ('quit',K_q,self.parent),
                ])
         
    def paint(self,screen):
        if self.first:
            self.first = False
            return dialog.Pause(self.game,'get ready',self.state)
