## Title: Classic
## Author: Phil Hassey
## Description: Level generator.

import galcon
import random
rr = random.randrange
import math

import classic
import dialog
import mapfile

import diffs


class Level_(classic.Level):
    def __init__(self,game,stats,parent,data = None):
        self.game = game
        self.parent = parent
        self.stats = stats
        self.data = data
        
    def init(self):
        l = self
        
        l.options.hide_stats = 1
        l.options.strategy = 1
        l.options.upgrades = 0

        u = galcon.Player()
        u.name = 'human'
        u.n = 1
        #u.team = t1
        l.users.add(u)
        l.player = u
        
        for row in self.stats:
            n,planets,production,ships = row[0:4]
            if len(row) == 6:
                u = row[4](*row[5])
                for lvl,spd,dname in diffs.DIFFS:
                    if diffs.get_diff() == lvl:
                        u.speed = int(spd*FPS)
                u.name = 'computer'# %d'%(n-1)
                u.n = n
                l.users.add(u)
        
        if self.data == None:
            self.generate()
            self.data = mapfile.tostring(self)
        else:
            mapfile.fromstring(self,self.data)
            
        self.sync()
            
        # initialize all the users / bots
        for u in self.users.values():
            u.level = self
            u.init()
            
        # misc
        self.timer,self.frame,self.first = 0,0,True
        
    def copy(self):
        return Level(self.game,self.stats,self.parent,self.data)
    
    def generate(self):
        homes = []
        l = self
        
        t_planets = 0
        n_total = 0
        for row in self.stats:
            n,planets,production,ships = row[0:4]
            if len(row) == 6:
                u = l.users[n]
            if production == None:
                n_total += planets
            else:
                for i in xrange(0,planets):
                    p_ = production
                    s_ = ships
                    if n in (2,4):
                        s_ = s_*diffs.get_diff()/100
                    p = galcon.Planet()
                    p.production = p_
                    p.ships = s_
                    p.user = l.users[n]
                    l.planets.add(p)
                    homes.append(p)

                    t_planets += 1

        t_planets += n_total
        scale = 1.0/((float(t_planets)/23.0)**0.5)
        l.options.scale = scale

        # add random planets
        for n in xrange(0,n_total):
            p = galcon.Planet()
            p.production = rr(0,100)
            p.ships = rr(0,50)
            p.user = self.users[0]
            self.planets.add(p)
        
        # set the scale - this formula works good
        self.options.scale = 1.0/((len(self.planets)/23.0)**0.5)
        
        # running this function syncs the data here with galcon
        # and gets valuable stuff like planet radius figured out, etc
        self.sync()
        
        # get some basic gist of the screen sizing ...
        sz = 16
        top = galcon.BORDER+sz
        left = galcon.PADDING+sz

        # randomize the position of the planets
        for p in self.planets.values():
            p.pos = rr(left+p.r,galcon.SW-p.r-left),rr(top+p.r,galcon.SH-p.r-top)

        # place the home planets in a circular formation
        SW,SH = galcon.SW,galcon.SH
        random.shuffle(homes)
        a = rr(0,360)
        inc = 360/len(homes)
        for p in homes:
            x = SW/2+(SH/2)*math.cos(a*math.pi*2.0/360.0)
            y = SH/2+(SH/2)*math.sin(a*math.pi*2.0/360.0)
            p.pos = x,y
            a += inc
            
        self.sync(True) # makes sure no planets are overlapping



def Level(game,stats,parent,data=None):
    l_ = Level_(game,stats,parent,data)
    l = galcon.Level(l_)
    l_.state = l
    return l
    
