## Title: Nibbles
## Author: Phil Hassey
## License: LGPL (c) 2007 Phil Hassey
## Description: The nibbles game code.

import galcon
import random
import dialog

SW,SH = 640,480
TW,TH = 8,8
W,H = 80,50
IX,IY = (SW-(W*TW))/2,(SH-(H*TH))/2
INIT = 12
EXT = 12
WIN = INIT+EXT*8
COLORS = [(0,0,0),(0x55,0x55,0xff),(0xff,0x55,0x55),(0xff,0xff,0x55),(0x55,0xff,0x55)]

WALL = 3
NIB = 4

LEVELS = ['1.txt','2.txt','3.txt','4.txt','5.txt','6.txt','7.txt','8.txt','9.txt']

def sign(v):
    if v < 0: return -1
    if v > 0: return 1
    return 0

class Level(galcon.State):
    def __init__(self,game,parent):
        self.game,self.parent = game,parent
        self.players = self.game.players
        
    def init(self):
        galcon.parts_init()
        self.part = galcon.Image('part.png')
        self.bkgr = galcon.Bkgr()
        self.font = self.game.font
        
        self.fname = LEVELS[self.game.cur%len(LEVELS)]
        
        l = self.layer = [[0 for x in xrange(W)] for y in xrange(0,H)]
        data = galcon.file_load(self.fname)
        y = 0
        for line in data.split('\n'):
            x = 0
            for c in line:
                l[y][x] = int((c == '#'))*WALL
                x += 1
            y += 1
        self.calc_lines()
            
        self.count = 8
        self.new_nib()
        
        for p in self.players:
            p.tail = []
            self.new_pos(p)
            p.timer = FPS*2
        
        self.frame = 0
    
    def new_nib(self):
        l = self.layer
        self.nib = x,y = self.find()
        l[y][x] = NIB
        
    def new_pos(self,p):
        l = self.layer
        x,y = p.pos = self.find(8)
        l[y][x] = p.n
        p.tail.append((x,y))
        p.length = INIT
        p.facing = self.getfacing(p.pos)
        p.q = []
        p.timer = FPS 
        
    def getfacing(self,pos):
        l = self.layer
        x,y = pos
        score = []
        for dx,dy in [(-1,0),(1,0),(0,-1),(0,1)]:
            d = 1
            while l[y+d*dy][x+d*dx] == 0: d += 1
            score.append((d,dx,dy))
        score.sort()
        d,dx,dy = score.pop()
        return dx,dy
        
    def find(self,b=2):
        l = self.layer
        v = 1
        while v != 0:
            x,y = random.randrange(b,W-b*2),random.randrange(b,H-b*2)
            v = l[y][x]
        return x,y
        
    def draw_box(self,screen,c,rect,w):
        x1,y1 = rect.x,rect.y
        x2,y2 = rect.right,rect.bottom
        screen.line(c,(x1,y1),(x1,y2),w)
        screen.line(c,(x1,y2),(x2,y2),w)
        screen.line(c,(x2,y2),(x2,y1),w)
        screen.line(c,(x2,y1),(x1,y1),w)
        
    def calc_lines(self):
        l = [v[:] for v in self.layer] # dupe the layer
        self.lines = []
        for y1 in xrange(0,H):
            for x1 in xrange(0,W):
                v = l[y1][x1]
                if v not in (WALL,): continue
                x2,y2 = x1,y1
                if x2 == 0 or l[y2][x2-1] != v:
                    while x2 < W-1 and l[y2][x2+1] == v: x2 += 1
                    if x2 != x1: self.lines.append((v,(x1,y1),(x2,y2)))
                x2,y2 = x1,y1
                if y2 == 0 or l[y2-1][x2] != v:
                    while y2 < H-1 and l[y2+1][x2] == v: y2 += 1
                    if y2 != y1: self.lines.append((v,(x1,y1),(x2,y2)))
        
    def paint(self,screen):
        screen.blit(self.bkgr,(0,0))
        
        l = self.layer
        # boring looking box rendering !!
        #for y in xrange(0,H):
            #for x in xrange(0,W):
        #for y in xrange(1,H-1):
            #for x in xrange(1,W-1):
                #v = l[y][x]
                #if v in (NIB,):
                    #screen.fill(COLORS[v],(IX+x*TW,IY+y*TH,TW,TH))
        #v = NIB
        #x,y = self.nib
        #screen.fill(COLORS[v],(IX+x*TW,IY+y*TH,TW,TH))
        c = COLORS[NIB]
        x,y = self.get_coord(self.nib)
        screen.circle(c,(x,y),3,5)
        k = FPS/2
        n = self.frame %(k*2)
        if n > k: n = k*2 - n
        c = c[0]*n/k,c[1]*n/k,c[2]*n/k
        screen.circle(c,(x,y),12,4)
                    
        for v,p1,p2 in self.lines:
            c = COLORS[v] ; x1,y1 = p1 ; x2,y2 = p2 
            screen.line(c,(IX+x1*TW+TW/2,IY+y1*TH+TH/2),(IX+x2*TW+TW/2,IY+y2*TH+TH/2),6)
                    
        #c = COLORS[WALL]
        #x1 = IX + TW/2
        #y1 = IY + TH/2
        #x2 = SW - IX - TW/2
        #y2 = SH - IY - TH/2
        #self.draw_box(screen,c,galcon.Rect(x1,y1,x2-x1,y2-y1),3)
        
        for p in self.players:
            # render the head of the player
            c = COLORS[p.n]
            x,y = self.get_coord(p.pos)
            if p.timer:
                screen.circle(c,(x,y),12,4)
                
                #if len(p.tail) == 1:
                screen.circle(c,(x,y),4,4)
                
            # render the score
            c = (255,255,255)
            text = '%05d'%p.score
            img = self.font.render(text,1,c)
            pad = 8
            x,y = pad,(IY-img.get_height())/2
            if p.n == 2:
                x = SW-img.get_width()-pad
            screen.blit(img,(x,y))
            
            # render that crazy trail
            c = COLORS[p.n]
            dx,dy = 0,0
            x1,y1 = p.tail[0]
            px,py = x1,y1
            for x2,y2 in p.tail:
                mv = abs(x2-px)+abs(y2-py)
                if mv == 1:
                    mx,my = sign(x2-x1),sign(y2-y1)
                    if (mx,my) != (dx,dy):
                        screen.line(c,(IX+x1*TW+TW/2,IY+y1*TH+TH/2),(IX+px*TW+TW/2,IY+py*TH+TH/2),6)
                        screen.line(c,(IX+x1*TW+TW/2,IY+y1*TH+TH/2),(IX+px*TW+TW/2,IY+py*TH+TH/2),2)
                        x1,y1 = px,py
                        dx,dy = sign(x2-x1),sign(y2-y1)
                    px,py = x2,y2
                else:
                    screen.line(c,(IX+x1*TW+TW/2,IY+y1*TH+TH/2),(IX+px*TW+TW/2,IY+py*TH+TH/2),6)
                    screen.line(c,(IX+x1*TW+TW/2,IY+y1*TH+TH/2),(IX+px*TW+TW/2,IY+py*TH+TH/2),2)
                    x1,y1 = x2,y2
                    dx,dy = 0,0
                    
            screen.line(c,(IX+x1*TW+TW/2,IY+y1*TH+TH/2),(IX+x2*TW+TW/2,IY+y2*TH+TH/2),6)
            screen.line(c,(IX+x1*TW+TW/2,IY+y1*TH+TH/2),(IX+x2*TW+TW/2,IY+y2*TH+TH/2),2)
                    
                
        
        c = (255,255,255)
        text = 'Nibbles - Round %02d'%(self.game.cur+1)
        img = self.font.render(text,1,c)
        x,y = (SW-img.get_width())/2,(IY-img.get_height())/2
        screen.blit(img,(x,y))
        
        c = (255,255,255)
        if len(self.players) == 1:
            text = 'use arrows to play'
        else:
            text = 'blue - use arrows        red - use W/A/S/D'
        img = self.font.render(text,1,c)
        x,y = (SW-img.get_width())/2,SH-IY+(IY-img.get_height())/2
        screen.blit(img,(x,y))
        
        galcon.parts_paint(screen)
        
        ##s1.fill((0,0,0),(0,400,640,80))
        #fnt = self.font
        
        #img = fnt.render("SCORE: %05d"%(self.score),1,(255,255,255))
        #sw = 640
        #s1.blit(img,((sw-img.get_width())/2,400+(80-img.get_height())/2))
        
        #if self.cur != 0:
            #img = fnt.render("press W to wimp out",1,(0xaa,0xaa,0xaa))
            #s1.blit(img,((sw-img.get_width())/2,480-img.get_height()))

        #if self.frame == 0:
            #return states.Pause(self.game,'GET READY!',self)
        
    def event(self,e):
        for p in self.players:
            d = None
            if e.type is KEYDOWN and e.key == p.k_up: d = (0,-1)
            if e.type is KEYDOWN and e.key == p.k_right: d = (1,0)
            if e.type is KEYDOWN and e.key == p.k_down: d = (0,1)
            if e.type is KEYDOWN and e.key == p.k_left: d = (-1,0)
            if d != None:
                if p.timer: p.q = []
                p.q.append(d)
        if e.type is KEYDOWN and e.key == K_ESCAPE:
            return dialog.Prompt(self.game,'Quit?',self.parent,self) 

        if e.type is KEYDOWN and e.key == K_PAGEUP:
            self.game.cur += 1
            return Level(self.game,self.parent)
        
        if e.type is KEYDOWN and e.key == K_PAGEDOWN:
            self.game.cur = max(0,self.game.cur-1)
            return Level(self.game,self.parent)
            
            
    def get_coord(self,pos):
        x,y = pos
        return IX+x*TW+TW/2,IY+y*TH+TH/2
        
    def loop(self):
        l = self.layer
        
        if not self.count:
            self.game.cur += 1
            return Level(self.game,self.parent)
        
        for p in self.players:
            tail = p.tail
            if (self.frame%2):
                if (p.timer and len(tail) > 1) or len(tail) > p.length:
                    x,y = tail.pop(0)
                    l[y][x] = 0
            
            if p.timer and len(tail) == 1:
                p.timer -= 1
            if p.timer:
                continue
            
            if not (self.frame%2):
                continue
            
            x,y = p.pos
        
            if len(p.q):
                f = p.q.pop(0)
                if (-f[0],-f[1]) != p.facing:
                    p.facing = f
            x,y = p.pos = p.pos[0]+p.facing[0],p.pos[1]+p.facing[1]
            
            v = l[y][x]
            
            if v and v != NIB:
                galcon.parts_add(self.part,self.get_coord(p.pos),8,128)
                self.new_pos(p)
                continue
            
            l[y][x] = p.n
            tail.append((x,y))
            
            if v == NIB:
                p.score += p.length
                p.length += EXT
                self.count -= 1
                galcon.parts_add(self.part,self.get_coord(p.pos),8,128)
                if self.count:
                    self.new_nib()
                
        
        galcon.parts_loop()
        self.frame += 1

