## Title: Classic
## Author: Phil Hassey
## License: Public Domain

import galcon
import random
rr = random.randrange
import math

import bot
import classic
import dialog
import mapfile

class Level_(classic.Level):
    def __init__(self,game,opts,parent,data = None):
        self.game = game
        self.opts = opts
        self.parent = parent
        self.data = data
        
    def init(self):
        # set up the player
        u = galcon.Player()
        u.name = 'player'
        self.users.add(u)
        self.player = u

        for n in xrange(0,self.opts['bots']):
            # set up the bot
            u = bot.Bot(self)
            u.name = 'bot %d'%(n+1)
            self.users.add(u)
        
        if self.data == None:
            self.generate()
            self.data = mapfile.tostring(self)
        else:
            mapfile.fromstring(self,self.data)
            
        # initialize all the users / bots
        for u in self.users.values():
            u.init()
            
        # misc
        self.timer,self.frame,self.first = 0,0,True
        
    def copy(self):
        return Level(self.game,self.opts,self.parent,self.data)
    
    def generate(self):
        # a list of home planets
        homes = []
        
        for i in xrange(0,self.opts['homes']):
            # set up the player's home planet
            u = self.users[1]
            p = galcon.Planet()
            p.production = self.opts['production']
            p.ships = self.opts['ships']
            p.user = u
            self.planets.add(p)
            homes.append(p)
            
            for j in xrange(0,self.opts['bots']):
                # set up the bot's home planet
                u = self.users[2+j]
                p = galcon.Planet()
                p.production = self.opts['production']
                p.ships = self.opts['ships']
                p.user = u
                self.planets.add(p)
                homes.append(p)
        
        # add 21 random planets
        for n in xrange(0,self.opts['planets']):
            p = galcon.Planet()
            p.production = rr(0,100)
            p.ships = rr(0,50)
            p.user = self.users[0]
            self.planets.add(p)
        
        # set the scale - this formula works good
        self.options.scale = 1.0/((len(self.planets)/23.0)**0.5)
        
        # running this function syncs the data here with galcon
        # and gets valuable stuff like planet radius figured out, etc
        self.sync()
        
        # get some basic gist of the screen sizing ...
        sz = 16
        top = galcon.BORDER+sz
        left = galcon.PADDING+sz

        # randomize the position of the planets
        for p in self.planets.values():
            p.pos = rr(left+p.r,galcon.SW-p.r-left),rr(top+p.r,galcon.SH-p.r-top)

        # place the home planets in a circular formation
        SW,SH = galcon.SW,galcon.SH
        random.shuffle(homes)
        a = rr(0,360)
        inc = 360/len(homes)
        for p in homes:
            x = SW/2+(SH/2)*math.cos(a*math.pi*2.0/360.0)
            y = SH/2+(SH/2)*math.sin(a*math.pi*2.0/360.0)
            p.pos = x,y
            a += inc

        self.sync(True) # makes sure no planets are overlapping

def Level(game,parent,opts,data=None):
    l_ = Level_(game,parent,opts,data)
    l = galcon.Level(l_)
    l_.state = l
    return l
    