## Title: Classic
## Author: Phil Hassey
## License: Public Domain
## Description: Level generator.

import galcon
import random
rr = random.randrange
import math

import bot
import classic
import dialog
import mapfile

class Level_(classic.Level):
    def __init__(self,game,parent,data = None):
        self.game = game
        self.parent = parent
        self.data = data
        
    def init(self):
        # set up the player
        u = galcon.Player()
        u.name = 'player'
        self.users.add(u)
        self.player = u

        # set up the bot
        u = bot.Bot(self)
        u.name = 'bot'
        self.users.add(u)
        
        if self.data == None:
            self.generate()
            self.data = (mapfile.tostring(self),self.get_ext())
        else:
            mf,ext = self.data
            mapfile.fromstring(self,mf)
            self.set_ext(ext)
            
        # initialize all the users / bots
        for u in self.users.values():
            u.init()
            
        # misc
        self.timer,self.frame,self.first = 0,0,True
        
    def copy(self):
        return Level(self.game,self.parent,self.data)
    
    def get_ext(self):
        r = []
        for p in self.moons:
            r.append((p.n,p.orbit.n,p.o_r,p.o_a,p.o_s))
        return r
        
    def set_ext(self,r):
        self.moons = []
        for n,o_n,o_r,o_a,o_s in r:
            p = self.planets[n]
            p.orbit = self.planets[o_n]
            p.o_r = o_r
            p.o_a = o_a
            p.o_s = o_s
            self.moons.append(p)
            self.orbit(p)
    
    def generate(self):
        # a list of home planets
        homes = []
        
        # set up the player's home planet
        u = self.users[1]
        p = galcon.Planet()
        p.production = 100
        p.ships = 100
        p.user = u
        self.planets.add(p)
        homes.append(p)
        
        # set up the bot's home planet
        u = self.users[2]
        p = galcon.Planet()
        p.production = 100
        p.ships = 100
        p.user = u
        self.planets.add(p)
        homes.append(p)
        
        # add 21 random planets
        self.moons = []
        for n in xrange(0,8):
            p = galcon.Planet()
            p.production = rr(0,100)
            p.ships = rr(0,50)
            p.user = self.users[0]
            self.planets.add(p)
            
        
            p2 = galcon.Planet()
            p2.production = rr(0,100)
            p2.ships = rr(0,50)
            p2.user = self.users[0]
            p2.orbit = p
            self.moons.append(p2)
        
        # set the scale - this formula works good
        self.options.scale = 1.0/((len(self.planets)*2/23.0)**0.5)
        
        # running this function syncs the data here with galcon
        # and gets valuable stuff like planet radius figured out, etc
        self.sync()
        
        # get some basic gist of the screen sizing ...
        sz = 16
        top = galcon.BORDER+sz
        left = galcon.PADDING+sz

        # randomize the position of the planets
        for p in self.planets.values():
            p.pos = rr(left+p.r,galcon.SW-p.r-left),rr(top+p.r,galcon.SH-p.r-top)

        # place the home planets in a circular formation
        SW,SH = galcon.SW,galcon.SH
        random.shuffle(homes)
        a = rr(0,360)
        inc = 360/len(homes)
        for p in homes:
            x = SW/2+(SH/2)*math.cos(a*math.pi*2.0/360.0)
            y = SH/2+(SH/2)*math.sin(a*math.pi*2.0/360.0)
            p.pos = x,y
            a += inc
        
        self.sync(True) # makes sure no planets are overlapping
            
        # get them orbits working
        for p in self.moons:
            p.o_r = rr(50,100)
            p.o_a = rr(0,360)
            p.o_s = (rr(1,5)*(rr(0,2)*2-1))/5.0
            self.orbit(p)
            self.planets.add(p)
            
    def loop(self):
        classic.Level.loop(self)
        for p in self.moons: self.orbit(p)

    def orbit(self,p):
        p.o_a += p.o_s
        pos = p.orbit.pos
        a = p.o_a * 2 * math.pi / 360
        p.pos = pos[0]+math.sin(a)*p.o_r,pos[1]+math.cos(a)*p.o_r

def Level(game,parent,data=None):
    l_ = Level_(game,parent,data)
    l = galcon.Level(l_)
    l_.state = l
    return l
    