## Title: mapfile.py
## Author: Phil Hassey
## License: LGPL (c) 2007 Phil Hassey
## Description: Map serialization tools.

import galcon

def tostring(l):
    lines = []
    lines.append('\t'.join(pack('version',{
        'value':1,
        })))
    lines.append('\t'.join(pack('options',{
        'scale':l.options.scale,
        })))
    for p in l.planets.values():
        lines.append('\t'.join(pack('planet',{
            'n':p.n,
            'pos_x':p.pos[0],
            'pos_y':p.pos[1],
            'production':p.production,
            'ships':p.ships,
            'user_n':p.user.n,
            'name':p.name,
            })))
    return '\n'.join(lines)


def fromstring(l,data):
    for line in data.split('\n'):
        vals = [v for v in line.split('\t')]
        t = vals[0]
        if t == 'version':
            pass
        elif t == 'options':
            e = unpack(vals)
            l.options.scale = e['scale']
        elif t == 'planet':
            e = unpack(vals)
            p = galcon.Planet()
            p.n = e['n']
            p.pos = e['pos_x'],e['pos_y']
            p.production, p.ships = e['production'],e['ships']
            p.user = l.users[e['user_n']]
            p.name = e['name']
            l.planets.add(p)

def pack(t,e):
    vs = [t]
    for typ,k,d in FORMAT[t]:
        vs.append(str(typ(e[k])))
    return vs

def unpack(e):
    t = e[0]
    e = e[1:]
    r = {}
    for n in xrange(0,len(FORMAT[t])):
        fld = FORMAT[t][n]
        typ,k,d = fld
        if len(e) > n:
            r[k] = typ(e[n])
        else:
            r[k] = d
    return r

FORMAT = {
    'version':[
        (int,'value',0),
        ],
    'options':[
        (float,'scale',1.0),
        ],
    'planet':[
        (int,'n',0),
        (int,'pos_x',0),
        (int,'pos_y',0),
        (int,'production',0),
        (int,'ships',0),
        (int,'user_n',0),
        (str,'name',''),
        ],
    }
