## Title: Classic
## Author: Phil Hassey
## License: Public Domain

import galcon
import random
rr = random.randrange
import math

import bot
import classic
import dialog
import mapfile

class Level_(classic.Level):
    def __init__(self,game,parent,data = None):
        self.game = game
        self.parent = parent
        self.data = data
        
    def init(self):
        ####################################################
        # create some teams
        t = galcon.Team()
        t.name = 'team 1' # creative naming at its best
        #t.color = COLORS[1] # team colors are optional ...
        self.teams.add(t)
        t = galcon.Team()
        t.name = 'team 2' # more of the same
        t.color = COLORS[2]
        self.teams.add(t)
        ####################################################
        
        # set up the player
        u = galcon.Player()
        u.name = 'player'
        self.users.add(u)
        self.player = u
        u.team = self.teams[1] ## <--

        # set up the bots
        u = bot.Bot(self)
        u.name = 'bot 1'
        self.users.add(u)
        u.team = self.teams[1] ## <--
        
        # set up the bots
        u = bot.Bot(self)
        u.name = 'bot 2'
        self.users.add(u)
        u.team = self.teams[2] ## <--
        
        # set up the bots
        u = bot.Bot(self)
        u.name = 'bot 3'
        self.users.add(u)
        u.team = self.teams[2] ## <--
        
        if self.data == None:
            self.generate()
            self.data = mapfile.tostring(self)
        else:
            mapfile.fromstring(self,self.data)
            
        # initialize all the users / bots
        for u in self.users.values():
            u.init()
            
        # misc
        self.timer,self.frame,self.first = 0,0,True
        
    def copy(self):
        return Level(self.game,self.parent,self.data)
    
    def generate(self):
        # a list of home planets
        homes = []
        
        for u in self.users.values():
            if u.n == 0: continue
            p = galcon.Planet()
            p.production = 100
            p.ships = 100
            p.user = u
            self.planets.add(p)
            homes.append(p)
        
        # add 21 random planets
        for n in xrange(0,21):
            p = galcon.Planet()
            p.production = rr(0,100)
            p.ships = rr(0,50)
            p.user = self.users[0]
            self.planets.add(p)
        
        # set the scale - this formula works good
        self.options.scale = 1.0/((len(self.planets)/23.0)**0.5)
        
        # running this function syncs the data here with galcon
        # and gets valuable stuff like planet radius figured out, etc
        self.sync()
        
        # get some basic gist of the screen sizing ...
        sz = 16
        top = galcon.BORDER+sz
        left = galcon.PADDING+sz

        # randomize the position of the planets
        for p in self.planets.values():
            p.pos = rr(left+p.r,galcon.SW-p.r-left),rr(top+p.r,galcon.SH-p.r-top)

        # place the home planets in a circular formation
        SW,SH = galcon.SW,galcon.SH
        random.shuffle(homes)
        a = rr(0,360)
        inc = 360/len(homes)
        for p in homes:
            x = SW/2+(SH/2)*math.cos(a*math.pi*2.0/360.0)
            y = SH/2+(SH/2)*math.sin(a*math.pi*2.0/360.0)
            p.pos = x,y
            a += inc
            
        self.sync(True) # makes sure no planets are overlapping

def Level(game,parent,data=None):
    l_ = Level_(game,parent,data)
    l = galcon.Level(l_)
    l_.state = l
    return l
    