import galcon
import random
rr = random.randrange
import math

import thirdbots
import colorscheme
import classic
import dialog
import mapfile

class Level_(classic.Level):
	def __init__(self,game,parent,handicap,data = None):
		self.help = [
			'---------- Imperium ----------',
			'',
			'Our enemy is overwhelmingly powerful.',
			'However, he moves ships about very inefficiently,',
			'and predictably attacks our largest planet.',
			'If we move quickly, distract him wherever possible,',
			'and encourage his ships to fly long distances,',
			'we can win.',
			'',
			'Tip:',
			'Pay attention to your opponent\'s strategy;',
			'sometimes it can be exploited.  "Shift-clicking",',
			'attacking a target from every planet rather',
			'than just nearby ones, is a common mistake.',
			'Another common mistake is to fail to change',
			'ship percent from the "50%" default.'
			]
		self.game = game
		self.parent = parent
		self.handicap = handicap
		self.data = data
		
	def init(self):
		# player
		u = galcon.Player()
		u.name = 'Player'
		u.color = colorscheme.human
		self.users.add(u)
		self.player = u

		# newbie
		u = thirdbots.Bot_Newbie(self)
		u.name = 'Imperium'
		u.color = colorscheme.newbie
		self.users.add(u)
		
		if self.data == None:
			self.generate()
			self.data = mapfile.tostring(self)
		else:
			mapfile.fromstring(self,self.data)
			
		self.sync()

		# initialize all the users / bots
		for u in self.users.values():
			u.init()
			
		# misc
		self.timer,self.frame,self.first = 0,0,True
		
	def copy(self):
		return Level(self.game,self.parent,self.handicap,self.data)
	
	def generate(self):
		# a list of home planets
		homes = []
		
		# set up the newbie's home planet
		u = self.users[2]
		p = galcon.Planet()
		p.production = 100
		p.ships = 90
		p.user = u
		self.planets.add(p)
		homes.append(p)
		
		# set up the extra home planet
		u = self.users[1]
		p = galcon.Planet()
		p.production = 100
		p.ships = 30 + 5*self.handicap
		p.user = u
		self.planets.add(p)
		homes.append(p)
		
		# set up the player's home planet
		u = self.users[1]
		p = galcon.Planet()
		p.production = 100
		p.ships = 30 + 5*self.handicap
		p.user = u
		self.planets.add(p)
		homes.append(p)
		
		for n in xrange(1,19):
			p = galcon.Planet()
			p.production = 5*n + rr(0,10)
			p.ships = n*2
			p.user = self.users[2]
			self.planets.add(p)
		for n in xrange(0,5):
			p = galcon.Planet()
			p.production = 20*n + rr(0,20)
			p.ships = 10*n + rr(0,10)
			p.user = self.users[0]
			self.planets.add(p)
		for n in xrange(0,5):
			p = galcon.Planet()
			p.production = 20*n + rr(0,20)
			p.ships = 10*(4-n) + rr(0,10)
			p.user = self.users[0]
			self.planets.add(p)
		
		# set the scale - this formula works good
		self.options.scale = 1.0/((len(self.planets)/23.0)**0.5)
		
		# running this function syncs the data here with galcon
		# and gets valuable stuff like planet radius figured out, etc
		self.sync()
		
		# get some basic gist of the screen sizing ...
		sz = 16
		top = galcon.BORDER+sz
		left = galcon.PADDING+sz

		# randomize the position of the planets
		for p in self.planets.values():
			p.pos = rr(left+p.r,galcon.SW-p.r-left),rr(top+p.r,galcon.SH-p.r-top)

		# place the home planets in a circular formation
		SW,SH = galcon.SW,galcon.SH
		random.shuffle(homes)
		a = rr(0,360)
		inc = 360/len(homes)
		for p in homes:
			x = SW/2+(SH/2)*math.cos(a*math.pi*2.0/360.0)
			y = SH/2+(SH/2)*math.sin(a*math.pi*2.0/360.0)
			p.pos = x,y
			a += inc


def Level(game,parent,difficulty=100,data=None):
	l_ = Level_(game,parent,difficulty,data)
	l = galcon.Level(l_)
	l_.state = l
	return l
	